/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

/// \page CPPTestApplications C++ Test Applications
/// The C++ test applications can be downloaded from www.roniza.com/downloads
/// \section CPPCreateImageExample C++ Crateing DICOM Image Example
/// C++ Crateing DICOM Image Example
/// This example shows how to use the DCXOBJ (DicomObject) in the
/// rzdcx.dll to create a DICOM Image with Pixel data

#include <stdio.h>
#include <time.h>
#include <string>
#include <list>

/// Dll import from the current directory
#import "../rzdcx.tlb" rename_namespace ("rzdcxLib")
using namespace rzdcxLib;
using namespace std;

static int ROWS=512;
static int COLUMNS=512;
static int BITS_ALLOCATED=12;
static int BITS_STORED=16;
static int RESCALE_INTERCEPT=0;
static int SAMPLES_PER_PIXEL=1;
static int NUMBER_OF_FRAMES=1;
static char* PHOTOMETRIC_INTERPRETATION="MONOCHROME2";

template<typename T>
static void CreateImage(char* filename)
{
    /// Create a DCXOBJ
    IDCXOBJPtr obj(__uuidof(DCXOBJ));

    /// Create an element pointer to place in the object for every tag
    IDCXELMPtr el(__uuidof(DCXELM));

	IDCXUIDPtr id(__uuidof(DCXUID));

	/// Set Hebrew Character Set
	el->Init(rzdcxLib::SpecificCharacterSet);
	el->Value = "ISO_IR 192";
    /// insert the element to the object
    obj->insertElement(el);

    /// You don't have to create an element every time, 
    /// just initialize it.
	char pn[]="John^Doe";
    el->Init(rzdcxLib::PatientsName);
    el->PutCStringPtr((int)pn);
    obj->insertElement(el);

	el->Init(rzdcxLib::patientID);
    el->Value = "123765";
    obj->insertElement(el);

	el->Init(rzdcxLib::studyInstanceUID);
	el->Value = id->CreateUID(UID_TYPE_STUDY);
	obj->insertElement(el);

	el->Init(rzdcxLib::seriesInstanceUID);
	el->Value = id->CreateUID(UID_TYPE_SERIES);
	obj->insertElement(el);
	
	el->Init(rzdcxLib::sopInstanceUID);
	el->Value = id->CreateUID(UID_TYPE_INSTANCE);
	obj->insertElement(el);

	el->Init(rzdcxLib::sopClassUid);
	el->Value = "1.2.840.10008.5.1.4.1.1.7"; // Secondary Capture
	obj->insertElement(el);

	el->Init(rzdcxLib::Rows);
	el->Value = ROWS;
	obj->insertElement(el);

	el->Init(rzdcxLib::Columns);
	el->Value = COLUMNS;
	obj->insertElement(el);

	el->Init(rzdcxLib::SamplesPerPixel);
	el->Value = SAMPLES_PER_PIXEL;
	obj->insertElement(el);

	el->Init(rzdcxLib::PhotometricInterpretation);
	el->Value = PHOTOMETRIC_INTERPRETATION;
	obj->insertElement(el);

	if (string("RGB") == PHOTOMETRIC_INTERPRETATION)
	{
		el->Init(rzdcxLib::PlanarConfiguration);
		el->Value = 0;
		obj->insertElement(el);
	}

	el->Init(rzdcxLib::BitsAllocated);
	el->Value = BITS_ALLOCATED;
	obj->insertElement(el);

	el->Init(rzdcxLib::BitsStored);
	el->Value = BITS_STORED;
	obj->insertElement(el);

	el->Init(rzdcxLib::HighBit);
	el->Value = BITS_STORED-1;
	obj->insertElement(el);

	el->Init(rzdcxLib::PixelRepresentation);
	el->Value = 0;
	obj->insertElement(el);

	el->Init(rzdcxLib::WindowCenter);
	el->Value = (int)(1<<(BITS_STORED-1));
	obj->insertElement(el);
	
	el->Init(rzdcxLib::WindowWidth);
	el->Value = (int)(1<<BITS_STORED);
	obj->insertElement(el);

	el->Init(rzdcxLib::RescaleIntercept);
	el->Value = (short)RESCALE_INTERCEPT;
	obj->insertElement(el);

	el->Init(rzdcxLib::RescaleSlope);
	el->Value = (short)1;
	obj->insertElement(el);

	if (NUMBER_OF_FRAMES > 1)
	{
		el->Init(rzdcxLib::NumberOfFrames);
		el->Value = (short)NUMBER_OF_FRAMES;
		obj->insertElement(el);
	}

	el->Init(rzdcxLib::GraphicData);
	el->Value = "456\\8934\\39843\\223\\332\\231\\100\\200\\300\\400";
	obj->insertElement(el);

	el->Init(rzdcxLib::PixelData);
	el->Length = ROWS*COLUMNS*SAMPLES_PER_PIXEL*NUMBER_OF_FRAMES;
	T* pixels=new T[ROWS*COLUMNS*NUMBER_OF_FRAMES];
	for (int n=0; n<NUMBER_OF_FRAMES; n++) {
		for (int y=0; y<ROWS;y++){
			for (int x=0;x<COLUMNS;x++){
				int i=x+COLUMNS*y*n;
				pixels[i]=((i)%(1<<BITS_STORED))-RESCALE_INTERCEPT;
			}
		}
	}
	if (sizeof(T)==1 || sizeof(T) == 3)
		el->ValueRepresentation = VR_CODE_OB;

	el->Value = (unsigned long)pixels;
	obj->insertElement(el);

	/// Save it as is
	obj->saveFile(filename);

	delete[] pixels;

	/// Encode as JPEG lossless
	obj->openFile(filename);
	obj->EncodeLosslessJpeg();
	string jfname1(filename);
	jfname1+=".jpeglossless.dcm";
	obj->saveFile(jfname1.c_str());

	/// Encode as JPEG lossy 0 quality
	obj->openFile(filename);
	obj->EncodeJpeg(0);
	string jfname4(filename);
	jfname4+=".jpeglossy_0.dcm";
	obj->saveFile(jfname4.c_str());

	/// Encode as JPEG lossy 100 (best) quality
	obj->openFile(filename);
	obj->EncodeJpeg(100);
	string jfname2(filename);
	jfname2+=".jpeglossy_100.dcm";
	obj->saveFile(jfname2.c_str());

	/// Encode as JPEG lossy 50 quality
	obj->openFile(filename);
	obj->EncodeJpeg(50);
	string jfname3(filename);
	jfname3+=".jpeglossy_50.dcm";
	obj->saveFile(jfname3.c_str());

	if (string("RGB") == PHOTOMETRIC_INTERPRETATION)
	{
		obj->openFile(filename);
		obj->MakePaletteColor();
		string pal(filename);
		pal+=".palette.dcm";
		obj->saveFile(pal.c_str());
	}
}

struct ColorPixel {
	char r;
	char g;
	char b;
	const ColorPixel& operator=(int value) {
		r = ((value+0)%3)*127;
		g = ((value+1)%3)*127;
		b = ((value+2)%3)*127;
		return *this;
	}
};

/// \example CPPDicomObjectExample.cpp
/// This is an example of how to use the IDCXELM interface and the IDCXOBJ interface to
/// load, save, create and modify DICOM Images.
int main(int argc, char* argv[])
{
    /// Initialize the dll (has to be performed in every thread)
    HRESULT hr = ::CoInitialize(NULL); 

    if (!SUCCEEDED(hr)) {
        ::CoUninitialize();
        return 1;
    }

	/// RGB Color Image
	ROWS=512;
	COLUMNS=512;
	BITS_ALLOCATED=8;
	BITS_STORED=8;
	RESCALE_INTERCEPT=0;
	SAMPLES_PER_PIXEL=3;
	PHOTOMETRIC_INTERPRETATION="RGB";
	CreateImage<ColorPixel>("color.dcm");

	/// Monochrome 16 bit image 512x512 pixesl
	ROWS=512;
	COLUMNS=512;
	BITS_ALLOCATED=16;
	BITS_STORED=16;
	RESCALE_INTERCEPT=0;
	SAMPLES_PER_PIXEL=1;
	PHOTOMETRIC_INTERPRETATION="MONOCHROME2";
	CreateImage<unsigned short>("gray16.dcm");

	/// Monochrome 16 bit image with rescale intercept
	ROWS=512;
	COLUMNS=512;
	BITS_ALLOCATED=16;
	BITS_STORED=16;
	RESCALE_INTERCEPT=-1024;
	SAMPLES_PER_PIXEL=1;
	PHOTOMETRIC_INTERPRETATION="MONOCHROME2";
	CreateImage<unsigned short>("gray16r-1024.dcm");

	/// Monochrome 12 bit image
	ROWS=512;
	COLUMNS=512;
	BITS_ALLOCATED=16;
	BITS_STORED=12;
	RESCALE_INTERCEPT=0;
	SAMPLES_PER_PIXEL=1;
	PHOTOMETRIC_INTERPRETATION="MONOCHROME2";
	CreateImage<unsigned short>("gray12.dcm");

	ROWS=512;
	COLUMNS=512;
	BITS_ALLOCATED=16;
	BITS_STORED=12;
	RESCALE_INTERCEPT=-1024;
	SAMPLES_PER_PIXEL=1;
	PHOTOMETRIC_INTERPRETATION="MONOCHROME2";
	CreateImage<unsigned short>("gray12r-1024.dcm");

	/// Monochrome 8 bit image
	ROWS=512;
	COLUMNS=512;
	BITS_ALLOCATED=8;
	BITS_STORED=8;
	RESCALE_INTERCEPT=0;
	SAMPLES_PER_PIXEL=1;
	PHOTOMETRIC_INTERPRETATION="MONOCHROME2";
	CreateImage<unsigned char>("gray8.dcm");

	/// RGB Color multi-frame image
	ROWS=512;
	COLUMNS=512;
	BITS_ALLOCATED=8;
	BITS_STORED=8;
	RESCALE_INTERCEPT=0;
	SAMPLES_PER_PIXEL=3;
	PHOTOMETRIC_INTERPRETATION="RGB";
	NUMBER_OF_FRAMES = 30;
	CreateImage<ColorPixel>("color_30_frames.dcm");

	::CoUninitialize();

	return 0; 
}

