/*
 * Copyright (C) 2011, Roni Zaharia
 */

#include "DICOMIZERAPI.h"

#import "../rzdcx.tlb" rename_namespace ("rzdcxLib")

using namespace rzdcxLib;

void readTextValue(FILE* fp, string& outValue)
{
	char buf[1024];
	fgets(buf, 1024, fp);
	outValue = buf;
}

void readFilesList(FILE* fp, list<string>& outValue)
{
	char buf[1024];
	while (fscanf(fp, "%[^|\n]|", buf) == 1)
		outValue.push_back(buf);
	fgets(buf,10,fp);
}

bool DICOMIZERAPI::ExamInfo::readText(const char* filename)
{
	FILE* fp=fopen(filename, "r");
	if (fp == NULL) {
		printf("Can't open file %s\n", filename);
		return false;
	}
	readTextValue(fp, AccessionNumber);
	readTextValue(fp, Date);		
	readTextValue(fp, PatientID);
	readTextValue(fp, FirstNameEng);
	readTextValue(fp, LastNameEng);		
	readTextValue(fp, FirstNameHeb);
	readTextValue(fp, LastNameHeb);		
	readTextValue(fp, FatherNameHeb);
	readTextValue(fp, ExamTypeEng);		
	readTextValue(fp, ExamTypeHeb);		
	readTextValue(fp, LoginDoctor);		
	readTextValue(fp, Manufecturer);	
	readFilesList(fp, images);
	readFilesList(fp, report);
}



DICOMIZERAPI::DICOMIZERAPI(const string& myAETitle) 
: _myAETitle(myAETitle)
{
    HRESULT hr = ::CoInitialize(NULL); 
	IDCXAPPPtr app(__uuidof(DCXAPP));
	app->StartLogging("c:\\rzdcx.log");
}

DICOMIZERAPI::~DICOMIZERAPI()
{
	::CoUninitialize();
}

DICOMIZERAPI::OpResult DICOMIZERAPI::SaveAsDICOM(
		ExamInfo& examInfo,
		string saveInDir
		)
{
	OpResult res;
	res.code = Successful;

	try {
		if (saveInDir[saveInDir.length()-1] != '\\')
			saveInDir.append("\\");

		/// Create a DCXOBJ
		IDCXOBJPtr obj(__uuidof(DCXOBJ));

		/// Create an element pointer to place in the object for every tag
		IDCXELMPtr el(__uuidof(DCXELM));

		IDCXUIDPtr id(__uuidof(DCXUID));

		/// Set Hebrew Character Set
		el->Init(rzdcxLib::SpecificCharacterSet);
		el->Value = "ISO_IR 138";
		/// insert the element to the object
		obj->insertElement(el);

		/// Construct the patient name
		string patientName(examInfo.LastNameEng);
		patientName.append("="); /// = is the lingual separator
		patientName.append(examInfo.LastNameHeb);
		patientName.append("^");
		patientName.append(examInfo.FirstNameEng);
		patientName.append("=");
		patientName.append(examInfo.FirstNameHeb);
		patientName.append("^^^");
		patientName.append(examInfo.FatherNameHeb);

		/// You don't have to create an element every time, 
		/// just initialize it.

		/// Patient Name
		el->Init(rzdcxLib::PatientsName);
		el->PutCStringPtr((int)patientName.c_str());
		obj->insertElement(el);

		/// Patient ID
		el->Init(rzdcxLib::patientID);
		el->Value = examInfo.PatientID.c_str();
		obj->insertElement(el);

		/// Patient Sex
		el->Init(rzdcxLib::PatientSex);
		el->Value = "M";
		obj->insertElement(el);

		/// Patient Birth Date
		el->Init(rzdcxLib::PatientBirthDate);
		el->Value = "19770109";
		obj->insertElement(el);

		/// Make it Secondary Capture
		el->Init(rzdcxLib::sopClassUid);
		el->Value = "1.2.840.10008.5.1.4.1.1.7"; // Secondary Capture
		obj->insertElement(el);

		el->Init(rzdcxLib::AccessionNumber);
		el->Value = examInfo.AccessionNumber.c_str();
		obj->insertElement(el);

		el->Init(rzdcxLib::StudyDate);
		el->Value = examInfo.Date.c_str();
		obj->insertElement(el);

		el->Init(rzdcxLib::StudyTime);
		obj->insertElement(el); /// Empty

		el->Init(rzdcxLib::PerformingPhysicianName);
		el->Value = examInfo.LoginDoctor.c_str();
		obj->insertElement(el);

		el->Init(rzdcxLib::ReferringPhysicianName);
		obj->insertElement(el); /// Empty

		el->Init(rzdcxLib::Manufacturer);
		el->Value = examInfo.Manufecturer.c_str();
		obj->insertElement(el);

		string studyDesc(examInfo.ExamTypeEng);
		studyDesc.append("=");
		studyDesc.append(examInfo.ExamTypeHeb);
		el->Init(rzdcxLib::StudyDescription);
		el->Value = studyDesc.c_str();
		obj->insertElement(el);

		/// Create new study instance UID
		el->Init(rzdcxLib::studyInstanceUID);
		el->Value = id->CreateUID(UID_TYPE_STUDY);
		obj->insertElement(el);

		/// Study ID
		el->Init(rzdcxLib::StudyID);
		obj->insertElement(el);

		/// Create new series instance UID
		el->Init(rzdcxLib::seriesInstanceUID);
		el->Value = id->CreateUID(UID_TYPE_SERIES);
		obj->insertElement(el);
	
		/// First series
		el->Init(rzdcxLib::SeriesNumber);
		el->Value=1;
		obj->insertElement(el);

		el->Init(rzdcxLib::ConversionType);
		el->Value = "WSD";
		obj->insertElement(el);

		char buf[16];
		int ni(1);
		for (Images::const_iterator i=examInfo.images.begin(); i!=examInfo.images.end(); i++, ni++)
		{
			/// Create new sop instance UID
			el->Init(rzdcxLib::sopInstanceUID);
			el->Value = id->CreateUID(UID_TYPE_INSTANCE);
			obj->insertElement(el);

			el->Init(rzdcxLib::InstanceNumber);
			el->Value = ni;
			obj->insertElement(el);

			obj->SetJpegFrames(i->c_str());
			string filename(saveInDir);
			sprintf_s<sizeof(buf)>(buf, "IMG%04d.dcm", ni);
			filename+=buf;
			obj->saveFile(filename.c_str());
		}


		/// Create new series instance UID
		el->Init(rzdcxLib::seriesInstanceUID);
		el->Value = id->CreateUID(UID_TYPE_SERIES);
		obj->insertElement(el);

		/// Second series
		el->Init(rzdcxLib::SeriesNumber);
		el->Value=2;
		obj->insertElement(el);

		int nr(1);
		for (ReportScreenshots::iterator i=examInfo.report.begin(); i!= examInfo.report.end(); i++, nr++)
		{
			/// Create new sop instance UID
			el->Init(rzdcxLib::sopInstanceUID);
			el->Value = id->CreateUID(UID_TYPE_INSTANCE);
			obj->insertElement(el);

			el->Init(rzdcxLib::InstanceNumber);
			el->Value = nr;
			obj->insertElement(el);

			obj->SetJpegFrames(i->c_str());
			string filename(saveInDir);
			sprintf_s<sizeof(buf)>(buf, "REP%04d.dcm", nr);
			filename+=buf;
			obj->saveFile(filename.c_str());
		}
	}
	catch (const _com_error& ex)
	{
		res.code = PermanentFailure;
		res.errorMessage = _bstr_t(ex.Description());
	}
	catch (...)
	{
		res.code = PermanentFailure;
		res.errorMessage = "Unknown error";
	}
	return res;
}

static string ScanDirectory(string dirStr)
{
	string filesList = "";
	char path[MAX_PATH];
	WIN32_FIND_DATAA fd;
	DWORD dwAttr = FILE_ATTRIBUTE_DIRECTORY;
	sprintf_s<sizeof(path)>( path, "%s\\*", dirStr.c_str());
	HANDLE hFind = FindFirstFileA( path, &fd);
	if(hFind != INVALID_HANDLE_VALUE)
	{
		int count = 0;
		do
		{
			if( !(fd.dwFileAttributes & dwAttr))
			{
				filesList += dirStr;
				filesList += "\\";
				filesList += fd.cFileName;
				filesList += ";";
			}
			//else  // Recursive scan
			//{
			//	if ((strcmp(fd.cFileName, ".") != 0) &&
			//		(strcmp(fd.cFileName, "..") != 0))
			//	{
			//		string dir = dirStr;
			//		dir += "\\";
			//		dir += fd.cFileName;
			//		filesList += ScanDirectoryManually(dir);
			//	}
			//}
		}while( FindNextFileA( hFind, &fd));
		FindClose( hFind);
	}
	return filesList;
}

DICOMIZERAPI::OpResult DICOMIZERAPI::SendFolder(
		DICOMApplication& destination, ///< The info of the destination DICOM Application
		string dirToSend ///< Path to the folder where the files are
		)
{
	OpResult res;
	res.code = Successful;

    /// Create the requester object to call send
	IDCXREQPtr requester(__uuidof(DCXREQ));

    /// File lists of succeeded and failed files
	BSTR succeededFilesList = NULL;
	BSTR failedFilesList =  NULL;
    try{
        /// Call the send method and send the files
		/// Get all files from folder
		string filesList(ScanDirectory(dirToSend));
		requester->Send(
			this->_myAETitle.c_str(), 
			destination.AETitle.c_str(),
			destination.HostOrIPAddress.c_str(),
			destination.port,
			filesList.c_str(),
			&succeededFilesList,
			&failedFilesList);
	
		_bstr_t failedList(failedFilesList, false);
		if (failedList.length() > 0)
		{
			res.errorMessage += "Failed files: ";
			res.errorMessage += failedList;
		}

		/// Don't forget to free the memory of out parameters!!!
		SysFreeString(succeededFilesList);
		SysFreeString(failedFilesList);
	}
	catch (_com_error& ex)
	{
		//if (ex.Error() & 0x0000FFFF == 
		/// \todo compile list of intermitend errors
		if ((ex.Error() & 0x0000ffff) == 0x0000031b) /// Timeout - can't connect
			res.code = IntermitentFailure;
		else
			res.code = PermanentFailure;

		res.errorMessage = _bstr_t(ex.Description());
	}
	catch (...)
	{
		res.code = PermanentFailure;
		res.errorMessage = "Unknown error";
	}
	return res;
}
