/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

/// \page CPPTestApplications C++ Test Applications
/// The C++ test applications can be downloaded from www.roniza.com/downloads
/// \section CPPEchoDemo C++ Echo Demo
/// C++ Echo Demo
/// This demo shows how to verify comunication with another Application Entity using the C-ECHO command

#include <stdio.h>
#include <time.h>
#include <string>
#include <list>
#include "atlconv.h"

/// Dll import from the current directory
#import "../rzdcx.tlb" rename_namespace ("rzdcxLib")
using namespace rzdcxLib;
using namespace std;

/// \example CPPEchoSCUExample.cpp 
/// ECHO SCU Example
/// Command line arguments are:
/// 1) LocalAE
/// 2) TargetAE
/// 3) Host (of target AE)
/// 4) Port (of target AE)
int main(int argc, char* argv[])
{
    /// See if we have all the params
    if (argc <= 4) return 1;

	HRESULT hr = ::CoInitialize(NULL); 

	_bstr_t e;
	bool failed(false);
	if (SUCCEEDED(hr))
    {
        /// Create a requester object to call the echo method
		IDCXREQPtr req(__uuidof(DCXREQ));
		try
        {
            /// Call echo
			req->Echo(
				argv[1], 
				argv[2], 
				argv[3],
				atoi(argv[4]));
			printf("Echo succeeded\n");
		}
        catch (_com_error& err)
        {
			failed=true;
			e = err.Description();
            /// If we have an exceptions, the Echo failed
			printf("Echo failed\n");
        }
        catch (...)
        {
            /// If we have an exceptions, the Echo failed
			printf("Unknown Error!\nEcho failed\n");
		}
	}

	if (failed) {
		USES_CONVERSION;
		printf("Message: %s\n", W2A(e));
	}

	::CoUninitialize();

	return 0;
}

