/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

#pragma once

#include "resource.h"

/// \brief The Console Event Manager captures console events by setting the appropriate hook for it.
/// It is used when the application is running not as a service to capture user inputs such as CTRL-C.
/// \ingroup DicomService
template <class T>
class ConsoleEventManager {
public:

	static HANDLE m_endEvent;

	ConsoleEventManager() {
		m_endEvent = CreateEvent( 
			NULL,		// default security attributes
			TRUE,		// manual-reset event
			FALSE,		// initial state is not signaled
			_T("EndEvent")	// object name
			); 

		if (m_endEvent == NULL) 
		{ 
			// Ok, Ok, Be Silent!
			//printf("CreateEvent failed (%d)\n", GetLastError());
			return;
		}
        SignalStarting();
		BOOL res = SetConsoleCtrlHandler(handler, TRUE);
		if (!res)
			printf("Failed to set console handler\n");
	}

	~ConsoleEventManager() {
        SignalStopping();
		BOOL res = SetConsoleCtrlHandler(handler, FALSE);
		if (!res)
			printf("Failed to unset console handler\n");
	}

	/// This function is called when console events are raised.
	/// In all cases but for logoff events, it stops the application.
	/// Since we don't know who logs off, this event is ignored.
	static BOOL WINAPI handler(DWORD control_code)
	{
		switch (control_code) 
		{
		case CTRL_C_EVENT:
		case CTRL_BREAK_EVENT:
		case CTRL_CLOSE_EVENT:
		case CTRL_SHUTDOWN_EVENT:
			T::Stop();
			printf("WaitForSingleObject returned %x\n", WaitForSingleObject(m_endEvent,30*1000));
			return TRUE;
		case CTRL_LOGOFF_EVENT:
			return FALSE;
		default:
			return FALSE;
		}
	}

	void SignalStopping() {
		BOOL res = SetEvent(m_endEvent);
	}
	void SignalStarting() {
		BOOL res = ResetEvent(m_endEvent);
	}

};


