/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

#pragma once

#include <string>

/// \page CPPTestApplications C++ Test Applications
/// \brief CSinkCommandTarget CSink Command Target
/// CSink Command Target
/// C++ Command Sink for connection points of the accepter class.
/// This class is provided as part of the C++ Storage SCP Demo Application.
/// Use this class to recieve callbacks of the Accepter class from C++ applications.
/// This class wraps the implementation of COM Connection points.
class CSink : public CCmdTarget
{
	DECLARE_DYNAMIC(CSink)

public:
	CSink();
	virtual ~CSink();

	/// Called when a Storage Commit result is recieved through a N-EVENT-REPORT command.
	HRESULT OnCommitResult(VARIANT_BOOL status, BSTR transactionUID, BSTR succeededInstances, BSTR failedInstances);

	/// Called for every incomming association request.
	/// Return true if you would like to accept the association or false to reject it.
	HRESULT OnConnection(BSTR calling, BSTR called, BSTR host, VARIANT_BOOL *accept_connection);

	/// Called when a C-STORE command starts. 
	/// filename is the filename of the stored object.
	HRESULT OnStoreSetup(BSTR *filename);
	
	/// Called when a C-STORE command ends. 
	/// filename is the filename of the stored object.
	HRESULT OnStoreDone(BSTR filename,
				VARIANT_BOOL storage_status,
				VARIANT_BOOL *accept_storage);

	/// Called when no commands has been recieved for a predefined period of time.
	HRESULT OnTimeout();

	virtual void OnFinalRelease();

    std::string mTID;
	std::string mSList; 
	std::string mFList; 

protected:
	DECLARE_MESSAGE_MAP()
	DECLARE_DISPATCH_MAP()
	DECLARE_INTERFACE_MAP()

private:
    BSTR mFilename;
};


