/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

/// \page CPPTestApplications C++ Test Applications
/// The C++ test applications can be downloaded from www.roniza.com/downloads
/// \section CPPQueryRetrieveSCUDemo C++ Query/Retrieve SCU Demo
/// C++ Query Retrieve SCU Demo
/// This example shows how to call Query to the target AE
/// on patients and how to retrieve the data (Move)

#include <stdio.h>
#include <time.h>
#include <string>
#include <list>
#include <atlbase.h>
#include "atlconv.h"

/// Dll import from the current directory
#import "../rzdcx.tlb" rename_namespace ("rzdcxLib")
using namespace rzdcxLib;
using namespace std;

/// \example CPPQueryRetrieveSCUExample.cpp
/// This example shows how to send a DICOM Query to the target AE
/// on patients and how to retrieve the data (Move)
/// 1) "Query" or "Move"
/// 2) LocalAE
/// 3) TargetAE
/// 4) Host
/// 5) Port
/// 6) AE to move to (only with Move) 
/// 7) Patient ID to move (only for Move)

int main(int argc, char* argv[])
{
    /// See if we have any command line args we recognise
    if (argc <= 1) return 1;

    /// If query requested
    if (strcmp(argv[1], "Query") == 0) { 
		HRESULT hr = ::CoInitialize(NULL); 

		if (SUCCEEDED(hr)){
            /// Create a requester object
			IDCXREQPtr req(__uuidof(DCXREQ));

            /// Create a DCXOBJ to fill it with the query data
			IDCXOBJPtr obj(__uuidof(DCXOBJ));
			
			IDCXELMPtr el(__uuidof(DCXELM));
			
			char* error=0;
			try
			{
                /// Query type - Patient
				el->Init(QueryRetrieveLevel);
				el->Value = "PATIENT";
				obj->insertElement(el);

                /// All query names
				el->Init(PatientsName);
				el->Value = "*";
				obj->insertElement(el);

                /// All query IDs
				el->Init(patientID);
				el->Value = "*";
				obj->insertElement(el);

                /// Call query method
				IDCXOBJIteratorPtr it = req->Query(argv[2], 
					argv[3], 
					argv[4],
					atoi(argv[5]),
					"1.2.840.10008.5.1.4.1.2.2.1", /// Query sop class uid
					obj);

                /// Iterate over the query results
				for (; !it->AtEnd(); it->Next())
				{
					IDCXOBJPtr currObj = it->Get();

					_variant_t ret;
					IDCXELMPtr elem = currObj->getElementByTag(0x00100010);
					if (elem != NULL)
					{
						ret = elem->Value;
					}

					string name = static_cast<_bstr_t>(ret);
                    printf("Patient name: %s\n", name.c_str());

					elem = currObj->getElementByTag(0x00100020);
					if (elem != NULL)
					{
						ret = elem->Value;
					}

					string id = static_cast<_bstr_t>(ret);
                    printf("Patient ID: %s\n", id.c_str());
				}

				printf("Query succeeded\n");
			}
			catch (_com_error& err)
			{
				printf("COM Error: %s\n", (char*)err.Description());
			}
			catch (...)
			{
				printf("Query failed");
				::CoUninitialize();
				return 1;
			}

		}

		::CoUninitialize();
        return 0; 
        
    /// If move requested
    } else if (strcmp(argv[1], "Move") == 0) { 
		HRESULT hr = ::CoInitialize(NULL); 

		if (SUCCEEDED(hr)){
            /// Create a requester object
			IDCXREQPtr req(__uuidof(DCXREQ));
			
            /// Create an object to load the file and request
            /// move on it's instance
			IDCXOBJPtr obj(__uuidof(DCXOBJ));

			try{
				req->MovePatient(  argv[2], 
								   argv[3], 
								   argv[4],
								   atoi(argv[5]),
								   argv[6],
								   "*",
                                   argv[7]);

				printf("Move succeeded\n");
			}
			catch (_com_error& err)
			{
				printf("COM Error: %s\n", (char*)err.Description());
			}
			catch (...)
			{
				printf("Move failed");
				::CoUninitialize();
				return 1;
			}
		}

		::CoUninitialize();
        return 0; 
    }
}

