/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

/// \page CPPTestApplications C++ Test Applications
/// The C++ test applications can be downloaded from www.roniza.com/downloads
/// \section CPPStorageSCPDemo C++ Storage SCP Demo
/// C++ Storage SCP Demo
/// This is a test application for the association accepter class.
/// The application listens on a port
/// The Sink class is the way to implement the event callbacks from
/// the dll.

#include "stdafx.h"
#include "CPPStorageSCPExample.h"
#include <stdio.h>
#include <time.h>
#include <afx.h>
#include <string>
#include "Sink.h"

using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

/// The one and only application object

CWinApp theApp;

using namespace std;

static bool isRunning = true;

struct Stopper 
{
    static void Stop() { isRunning = false; }
};

HANDLE ConsoleEventManager<Stopper>::m_endEvent;

/// \example CPPStorageSCPExample.cpp
/// Single Threaded DICOM Storage SCP Example
int main(int argc, char* argv[])
{
    // Create the console event manager so that if the
    // program was interrupted in the middle of the loop,
    // it will finish properly (Allways create it in the beggining
    // of the program)
    ConsoleEventManager<Stopper> cem;

    int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		printf("Fatal Error: MFC initialization failed\n");
		nRetCode = 1;
	}
	else
	{
		HRESULT hr = ::CoInitialize(NULL); 

		if (SUCCEEDED(hr)){
            /// Create the acceptor object for waiting for
            /// connections and commands
			IDCXACCPtr acc(__uuidof(DCXACC));

			if(acc)
			{
				IConnectionPointContainerPtr ptrConPntCnt = (IDispatch*)acc;
				IConnectionPointPtr ptrCon;
				IUnknown* pHandler = NULL;
				DWORD dwCookie;

				if(ptrConPntCnt != NULL)
				{
                    /// Connect the acceptor events to the Sink class that handles
                    /// them.
					ptrConPntCnt->FindConnectionPoint(__uuidof(IDCXACCEvents),&ptrCon);
			                    
					CSink *pSink = new CSink();

					pHandler = pSink->GetInterface(&IID_IUnknown);
					ptrCon->Advise(pHandler,&dwCookie);
			              
                    /// \code

					try{
                        /// This is the Local AE
						string ae = "Test";
						USES_CONVERSION;
						BSTR ae_title = A2BSTR(ae.c_str());

                        /// This is the way to use the acceptor to
                        /// run and wait for connections and commands.
						while (isRunning)
						{
							try
							{
                                /// Wait for the connection on port: 104,
                                /// timeout: 30 secs
								if (acc->WaitForConnection(ae_title, 104, 5))
								{
									/// If you want you an go to a new thread here
									bool res;
									do {
                                        /// Wait for a command from the client
										if (acc->WaitForCommand(30) == VARIANT_TRUE)
										{
											printf("%s %s %s", pSink->mTID.c_str(), pSink->mSList.c_str(), pSink->mFList.c_str());
											res = true;
										}
										else
										{
											res = false;
										}
									} while (res);
								}
							}
							catch (...)
							{
                                printf("store failed");
							}
                            /// If you want to sleep in the middle:
							/// Sleep(1000);
						}
						printf("store succeeded\n");
					}catch (...){
						printf("store failed");
					}

                    /// \endcode

                    /// Unattach the event callbacks.
					ptrCon->Unadvise(dwCookie);

					ptrCon.Release();
					ptrConPntCnt.Release();
					delete pSink;

				}
			}
		}
		
		::CoUninitialize();
	}

	return nRetCode;
}
