/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

/// \page CPPTestApplications C++ Test Applications
/// The C++ test applications can be downloaded from www.roniza.com/downloads
/// \section CPPStorageSCUDemo C++ Storage SCU Demo
/// C++ Storage SCU Demo
/// This example shows how to send DICOM files.
/// The application accept the list of file as command line arguments separated by a ; (smicolon)
/// To run the application simply call 
/// CPPStorageSCPExample local-ar remote-ae remote-host remote-port file1[;file2[;file3] ...]

#include <stdio.h>
#include <time.h>
#include <string>
#include <list>

/// Dll import from the current directory
#import "../rzdcx.tlb" rename_namespace ("rzdcxLib")
using namespace rzdcxLib;
using namespace std;

/// \example CPPStorageSCUExample.cpp
/// This example shows how to send DICOM files.
/// 1) LocalAE
/// 2) TargetAE
/// 3) Host
/// 4) Port
/// 5) ";" separated files list
int main(int argc, char* argv[])
{
    /// See if we have all the params
    if (argc <= 5) return 4;

    HRESULT hr = ::CoInitialize(NULL); 

	if (SUCCEEDED(hr)){ 

		IDCXAPPPtr app(__uuidof(DCXAPP));
		app->StartLogging("c:\\rzdcx.log");

        /// Create the requester object to call send
		IDCXREQPtr req(__uuidof(DCXREQ));

        /// File lists of succeeded and failed files
		BSTR succeededFilesList = NULL;
		BSTR failedFilesList =  NULL;
        try{
            /// Call the send method and send the files
			req->Send(argv[1],
					  argv[2],
					  argv[3],
					  atoi(argv[4]),
					  argv[5],
					  &succeededFilesList,
					  &failedFilesList);

			printf("Send succeeded\n");
			printf("Succedded files: %S\n", succeededFilesList);
			printf("Failed files: %S\n", failedFilesList);
			/// Don't forget to free the memory of out parameters!!!
			SysFreeString(succeededFilesList);
			SysFreeString(failedFilesList);
		}
		catch (_com_error& err)
		{
			printf("Send failed: %s\n", (char*)err.Description());
		}
		catch (...)
		{
			printf("Send failed");
			::CoUninitialize();

			return 1;
		}
	}

	::CoUninitialize();

	return 0;
}

