/// \page CSTestApplications C# Test Applications
/// The C# test applications can be downloaded from www.roniza.com/downloads
/// \section EchoSCUDemo Verification SCU Demo
/// Echo SCU Demo
/// This example shows how to call echo to the called AE.
/// In this project we use a manifest file instead of COM registration 
/// for demonstaration of registry free COM.
/// This technique is very usefull for example in DICOM CD Viewer where
/// you can't register the COM object on the CD but you would like to
/// use it in your application.

using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using rzdcxLib;
using System.Runtime.InteropServices;

/// \example EchoSCUExampleForm.cs
/// A C# DICOM Verification (C-ECHO) SCU
namespace EchoSCUExample
{
    public partial class EchoSCUExampleForm : Form
    {
        public EchoSCUExampleForm()
        {
            InitializeComponent();
        }

        private void EchoBtn_Click(object sender, System.EventArgs e)
        {
            DCXREQ req = null;
            try
            {
                // Create the requester object
                req = new DCXREQ();

                // Call the echo method. If echo fails an exception is thrown
                req.Echo(LocalAEEdit.Text,
                         TargetAEEdit.Text,
                         HostEdit.Text,
                         ushort.Parse(PortEdit.Text));

                MessageBox.Show("Echo succeeded");
            }
            catch (System.Runtime.InteropServices.COMException com_e)
            {
                MessageBox.Show("Echo failed: " + com_e.Message);
            }
            finally
            {
                ReleaseComObject(req);
            }
        }

        /* This has to be called for all the com objects to release the
           memory!!! */
        private void ReleaseComObject(object o)
        {
            if (o != null)
                Marshal.ReleaseComObject(o);
        }
    }
}