/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

/// \page CSTestApplications C# Test Applications
/// The C# test applications can be downloaded from www.roniza.com/downloads
/// \section MWLSCU Modality Worklist Query Demo
/// Modality Worklist Query Demo
/// This example shows how to make a Modality Worklist Query

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using rzdcxLib;
using System.Runtime.InteropServices;

/// \example ModalityWorklistSCUExampleForm.cs
/// A C# DICOM Query/Retrieve SCU Example
namespace ModalityWorklistSCU
{
    public partial class ModalityWorklistSCUExampleForm : Form
    {
        public ModalityWorklistSCUExampleForm()
        {
            InitializeComponent();
        }

        /* Callback when a response from the called ae for the 
           query command is recieved */
        public void OnQueryResponseRecievedAction(DCXOBJ obj)
        {
            // Do something
            ReleaseComObject(obj);
        }

        /* Callback when a response from the called ae for the 
           move command is recieved */
        public void OnMoveResponseRecievedAction(bool status, DCXOBJ obj)
        {
            // Do something
            ReleaseComObject(obj);
        }

        private void QueryBtn_Click(object sender, EventArgs e)
        {
            checkedListBox1.Items.Clear();

            DCXOBJIterator it = null;
            DCXREQ req = null;
            DCXOBJ obj = null;
            DCXOBJ sps = null;
            DCXELM el = null;
            DCXOBJIterator spsIt = null;
            try
            {
                // Fill the query object
                obj = new DCXOBJ();
                sps = new DCXOBJ();
                el = new DCXELM();
                
                // Build the SPS Sequence
                el.Init((int)DICOM_TAGS_ENUM.ScheduledStationAETitle);
                sps.insertElement(el);

                if (checkBox3.Checked)
                {
                    el.Init((int)DICOM_TAGS_ENUM.ScheduledProcedureStepStartDate);
                    el.Value = dateExact.Value;
                    sps.insertElement(el);
                }
                else
                {
                    if (checkBox2.Checked && checkBox1.Checked)
                    {
                        string dateRange = dateStartDate.Value.ToString("yyyyMMdd") + "-" + dateEndDate.Value.ToString("yyyyMMdd");
                        el.Value = dateRange;
                        sps.insertElement(el);
                    }
                    else if (checkBox2.Checked && !checkBox1.Checked)
                    {
                        string dateRange = dateStartDate.Value.ToString("yyyyMMdd") + "-";
                        el.Value = dateRange;
                        sps.insertElement(el);
                    }
                    else if (!checkBox2.Checked && checkBox1.Checked)
                    {
                        string dateRange = "-" + dateEndDate.Value.ToString("yyyyMMdd");
                        el.Value = dateRange;
                        sps.insertElement(el);
                    }
                }

                el.Init((int)DICOM_TAGS_ENUM.Modality);
                sps.insertElement(el);

                spsIt = new DCXOBJIterator();
                spsIt.Insert(sps);

                el.Init((int)DICOM_TAGS_ENUM.ScheduledProcedureStepSequence);
                el.Value = spsIt;
                obj.insertElement(el);


                /// Requested Procedure ID
                el.Init((int)DICOM_TAGS_ENUM.RequestedProcedureID);
                obj.insertElement(el);

                el.Init((int)DICOM_TAGS_ENUM.RequestedProcedureDescription);
                obj.insertElement(el);

                el.Init((int)DICOM_TAGS_ENUM.studyInstanceUID);
                obj.insertElement(el);

                el.Init((int)DICOM_TAGS_ENUM.PatientsName);
                obj.insertElement(el);

                el.Init((int)DICOM_TAGS_ENUM.patientID);
                obj.insertElement(el);



                // Create the requester object and connect it's callback to our method
                req = new DCXREQClass();
                req.OnQueryResponseRecieved += new IDCXREQEvents_OnQueryResponseRecievedEventHandler(OnQueryResponseRecievedAction);

                // send the query command
                it = req.Query(LocalAEEdit.Text, 
                               TargetAEEdit.Text, 
                               HostEdit.Text, 
                               ushort.Parse(PortEdit.Text),
                               "1.2.840.10008.5.1.4.31",  /// Modality Worklist SOP Class
                               obj);

                DCXOBJ currObj = null;

                try
                {
                    // Iterate over the query results
                    for (; !it.AtEnd(); it.Next())
                    {
                        currObj = it.Get();

                        string message = "";

                        message += "Req. Proc. ID: ";
                        message += currObj.getElementByTag((int)DICOM_TAGS_ENUM.RequestedProcedureID).Value.ToString();
                        message += "Req. Proc. Desc: ";
                        message += currObj.getElementByTag((int)DICOM_TAGS_ENUM.RequestedProcedureDescription).Value.ToString();
                        
                        checkedListBox1.Items.Add(message, false);
                    }
                }
                finally
                {
                    ReleaseComObject(currObj);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Query failed: " + ex.Message);
            }
            finally
            {
                ReleaseComObject(req);
                ReleaseComObject(obj);
                ReleaseComObject(it);
                ReleaseComObject(el);
                ReleaseComObject(spsIt);
            }
        }

        /* This has to be called for all the com objects to release the
           memory!!! */
        private void ReleaseComObject(object o)
        {
            if (o != null)
                Marshal.ReleaseComObject(o);
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1.Checked)
            {
                checkBox3.Checked = false;
                dateEndDate.Enabled = true;
            }
            else
                dateEndDate.Enabled = false;

        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox2.Checked)
            {
                checkBox3.Checked = false;
                dateStartDate.Enabled = true;
            }
            else
                dateStartDate.Enabled = false;

        }

        private void checkBox3_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox3.Checked)
            {
                dateExact.Enabled = true;
                checkBox1.Checked = false;
                checkBox2.Checked = false;
            }
            else
            {
                dateExact.Enabled = false;
            }
        }
    }
}