/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

/// \page CSTestApplications C# Test Applications
/// The C# test applications can be downloaded from www.roniza.com/downloads
/// \section QRSCU Query/Retrieve SCU Demo
/// Query/Retrieve SCU Demo
/// This example shows how to execute a query to the called ae
/// and how to retrieve dicom objects from the called ae to the 
/// target ae

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using rzdcxLib;
using System.Runtime.InteropServices;

/// \example QueryRetrieveSCUExampleForm.cs
/// A C# DICOM Query/Retrieve SCU Example
namespace QueryRetrieveSCU
{
    public partial class QueryRetrieveSCUExampleForm : Form
    {
        public QueryRetrieveSCUExampleForm()
        {
            InitializeComponent();
        }

        /* Callback when a response from the called ae for the 
           query command is recieved */
        public void OnQueryResponseRecievedAction(DCXOBJ obj)
        {
            throw new ApplicationException("Cancel!");
            DCXELM e = obj.getElementByTag((int)DICOM_TAGS_ENUM.patientID);
            string pid = e.Value.ToString();
            e = obj.getElementByTag((int)DICOM_TAGS_ENUM.PatientsBirthDate);
            string bdate;
            if (e != null && e.Value != null)
                bdate = e.Value.ToString();

            // Do something
            ReleaseComObject(obj);
        }

        /* Callback when a response from the called ae for the 
           move command is recieved */
        public void OnMoveResponseRecievedAction(bool status)
        {
            // Do something
            //ReleaseComObject(obj);
        }

        private void QueryBtn_Click(object sender, EventArgs e)
        {
            checkedListBox1.Items.Clear();

            DCXOBJIterator it = null;
            DCXREQ req = null;
            DCXOBJ obj = null;
            DCXELM el = null;
            try
            {
                // Fill the query object

                obj = new DCXOBJ();
                el = new DCXELM();

                el.Init((int)DICOM_TAGS_ENUM.QueryRetrieveLevel);
                el.Value = "PATIENT";
                obj.insertElement(el);

                el.Init(0x00100010);
                el.Value = PatientNameEdit.Text;
                obj.insertElement(el);

                el.Init(0x00100020);
                el.Value = PatientIDEdit.Text;
                obj.insertElement(el);

                el.Init((int)DICOM_TAGS_ENUM.PatientsSex);
                obj.insertElement(el);

                el.Init((int)DICOM_TAGS_ENUM.PatientsBirthDate);
                obj.insertElement(el);

                // Create the requester object and connect it's callback to our method
                req = new DCXREQClass();
                req.OnQueryResponseRecieved += new IDCXREQEvents_OnQueryResponseRecievedEventHandler(OnQueryResponseRecievedAction);

                // send the query command
                it = req.Query(LocalAEEdit.Text, 
                               TargetAEEdit.Text, 
                               HostEdit.Text, 
                               ushort.Parse(PortEdit.Text), 
                               "1.2.840.10008.5.1.4.1.2.1.1", 
                               obj);

                DCXOBJ currObj = null;

                try
                {
                    // Iterate over the query results
                    for (; !it.AtEnd(); it.Next())
                    {
                        currObj = it.Get();

                        string message = "";

                        DCXELM currElem = currObj.getElementByTag(0x00100020);
                        if (currElem != null)
                        {
                            message += "" + currElem.Value;
                        }

                        ReleaseComObject(currElem);

                        currElem = currObj.getElementByTag(0x00100010);
                        if (currElem != null)
                        {
                            message += " " + currElem.Value;
                        }

                        ReleaseComObject(currElem);

                        checkedListBox1.Items.Add(message, false);
                    }
                }
                finally
                {
                    ReleaseComObject(currObj);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Query failed: " + ex.Message);
            }
            finally
            {
                ReleaseComObject(req);
                ReleaseComObject(obj);
                ReleaseComObject(it);
                ReleaseComObject(el);
            }
        }

        /* This has to be called for all the com objects to release the
           memory!!! */
        private void ReleaseComObject(object o)
        {
            if (o != null)
                Marshal.ReleaseComObject(o);
        }

        private void MoveBtn_Click(object sender, EventArgs e)
        {
            DCXREQ req = null;
            try
            {
                // Create the requester object and connect it's event to our callback method
                req = new DCXREQClass();
                req.OnMoveResponseRecieved += new IDCXREQEvents_OnMoveResponseRecievedEventHandler(OnMoveResponseRecievedAction);

                string patientName = "";
                string patientID = "";
                char[] delimiter = {' '};
                for (int i = 0; i < checkedListBox1.CheckedItems.Count; i++)
                {
                    string temp = (string)checkedListBox1.CheckedItems[i];
                    string[] array = temp.Split(delimiter);

                    patientID = array[0];
                    patientName = array[1];

                    // Move the patient to the target ae
                    req.MovePatient(LocalAEEdit.Text,
                                   TargetAEEdit.Text,
                                   HostEdit.Text,
                                   ushort.Parse(PortEdit.Text),
                                   MoveToTargetAEEdit.Text,
                                   patientName,
                                   patientID);
                }

                MessageBox.Show("Move succeeded");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Move failed: " + ex.Message);
            }
            finally
            {
                ReleaseComObject(req);
            }
        }
    }
}