/*
 * Copyright (C) 2005 - 2011, Roni Zaharia
 */

/// \page CSTestApplications C# Test Applications
/// The C# test applications can be downloaded from www.roniza.com/downloads
/// \section StorageSCPDemo Storage SCP Demo
/// Storage SCP Demo
/// This demo shows how to use the acceptor class to wait for incomming
/// commands and process them

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using rzdcxLib;
using System.Runtime.InteropServices;

/// \example StorageSCPExampleForm.cs
/// A C# Single-Threaded DICOM Storage SCP Example
namespace StorageSCPExample
{
    public partial class StorageSCPExampleForm : Form
    {
        static private int _filecount = 0;

        public StorageSCPExampleForm()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            
        }

        /* Callback method when a connection is recieved
           Here you can check the calling ae and accept or 
           reject the association*/
        private void OnConnectionEventHandler(string callingTitle, 
                                       string calledTitle, 
                                       string callingHost,
                                       ref bool acceptConnection)
        {
            acceptConnection = true;
        }

        /* When a store command is recieved, you can provide the
           filename to store to*/
        private void OnStoreSetupEventHandler(ref string filename)
        {
            filename = ".\\dcm" + (++_filecount) + ".dcm";

            MessageBox.Show("Store setup on file: " + filename);
        }

        /* Handle the on timeout event here */
        private void OnTimeoutEventHandler()
        {
            MessageBox.Show("A timeout occurred");
        }

        /* When store is done, handle the file here and respond if storage went well */
        private void OnStoreDoneEventHandler(string filename, bool status, ref bool accept)
        {
            MessageBox.Show("Store done, file: " + filename + " status: " + status);

            string sop_instance_uid = null;
            DCXOBJ o = new DCXOBJ();
            o.openFile(filename);
            DCXELM e = o.getElementByTag((int)DICOM_TAGS_ENUM.sopInstanceUID);
            if (e != null)
            {
                sop_instance_uid = e.Value.ToString();
            }
            ReleaseComObject(e);
            ReleaseComObject(o);
            if (sop_instance_uid != null)
                System.IO.File.Move(filename, ".\\" + sop_instance_uid + ".dcm");

            accept = true;
        }

        /* When a result for the storage commit is recieved 
           this event occurs. Handle the succeeded and failed instances here*/
        private void OnCommitResultEventHandler(bool status, 
                                                string transactionUID, 
                                                string succeededInstances,
                                                string failedInstances)
        {
            MessageBox.Show("Commit result: Status: " + status + "\n" +
                            "TransactionUID: " + transactionUID + "\n" +
                            "SucceededInstances: " + succeededInstances + "\n" +
                            "FailedInstances: " + failedInstances);
        }

        private void ListenBtn_Click(object sender, EventArgs e)
        {

            DCXAPP app = new DCXAPP();
            app.StartLogging("c:\\rzdcxLog.txt");

            // Create the acceptor object here
            DCXACC acc = new DCXACC();

            // Connect all the events to the callback methods
            acc.OnConnection += new IDCXACCEvents_OnConnectionEventHandler(OnConnectionEventHandler);
            acc.OnStoreSetup += new IDCXACCEvents_OnStoreSetupEventHandler(OnStoreSetupEventHandler);
            acc.OnTimeout += new IDCXACCEvents_OnTimeoutEventHandler(OnTimeoutEventHandler);
            acc.OnStoreDone += new IDCXACCEvents_OnStoreDoneEventHandler(OnStoreDoneEventHandler);
            acc.OnCommitResult += new IDCXACCEvents_OnCommitResultEventHandler(OnCommitResultEventHandler);

            // This is the standard loop to wait for connections and commands
            while (true)
            {
                if (acc.WaitForConnection(LocalAEEdit.Text, ushort.Parse(PortEdit.Text), 30))
                {
                    // Can go to new thread
                    bool res;
                    do
                    {
                        if (acc.WaitForCommand(30))
                        {
                            res = true;
                        }
                        else
                        {
                            res = false;
                        }
                    } while (res);
                }
            }

            ReleaseComObject(acc);

            app.StopLogging();

            ReleaseComObject(app);
        }

        /* This has to be called for all the com objects to release the
           memory!!! */
        private void ReleaseComObject(object o)
        {
            if (o != null)
                Marshal.ReleaseComObject(o);
        }

    }
}