#include "ObjectAndElementTest.h"
#include <iostream>
#include <strstream>

#import "../rzdcx.tlb"  rename_namespace ("rzdcxLib")
using namespace rzdcxLib;
using namespace std;

CPPUNIT_TEST_SUITE_REGISTRATION( ObjectAndElementTest );

ObjectAndElementTest::ObjectAndElementTest()
{
}

ObjectAndElementTest::~ObjectAndElementTest()
{
}

void ObjectAndElementTest::setUp()
{
	HRESULT hr = ::CoInitialize(NULL); 
	CPPUNIT_ASSERT(SUCCEEDED(hr));
}

void ObjectAndElementTest::tearDown()
{
	::CoUninitialize();
}

template<typename T>
void TestSingleValue (T value, long tag)
{
	IDCXOBJPtr o(__uuidof(DCXOBJ));
	IDCXELMPtr in = o->createElement(tag);
	in->Value = value;
	o->insertElement(in);
	IDCXELMPtr out = o->getElementByTag(tag);
	CPPUNIT_ASSERT_EQUAL((T)in->Value, value);
	CPPUNIT_ASSERT_EQUAL((T)out->Value, value);
	CPPUNIT_ASSERT_EQUAL((T)in->Value, (T)out->Value);
}

template<typename T>
void TestMultiValue(T* values, unsigned long numValues, long tag)
{
	IDCXOBJPtr o(__uuidof(DCXOBJ));
	IDCXELMPtr in = o->createElement(tag);
	in->Length = numValues;
	in->Value = (unsigned long)values;
	o->insertElement(in);
	IDCXELMPtr out = o->getElementByTag(tag);
	CPPUNIT_ASSERT_EQUAL((long)numValues, out->ValueMultiplicity);
	_variant_t v = out->ValueArray;
	SAFEARRAY *pSafeArray = v.parray;
	T* dwArray = NULL;
	SafeArrayAccessData(pSafeArray, (void**)&dwArray);
	for (unsigned long i=0; i<numValues; i++)
	{
		CPPUNIT_ASSERT_EQUAL((T)dwArray[i], values[i]);
	}
	SafeArrayUnaccessData(pSafeArray);
}

/// Check that a SL (Signed Long) Value with VM = 1 works
void ObjectAndElementTest::Test_SingleValueSL ()
{
	TestSingleValue(15, rzdcxLib::SelectorSLValue);
}

long long_values[] = { -1024, 792 };

/// Check that a SL (Signed Long) Value with VM > 1 (Multi Value) works
void ObjectAndElementTest::Test_MultiValueSL ()
{
	TestMultiValue(long_values, sizeof(long_values)/sizeof(long_values[0]), rzdcxLib::DisplayedAreaTopLeftHandCorner);
}

void ObjectAndElementTest::Test_SingleValueFL ()
{
	TestSingleValue((float)-15.2, rzdcxLib::RecommendedDisplayFrameRateInFloat);
}

float float_values[] = { (float)1.2, (float)3.4, (float)5.6, (float)7.8, (float)9. };

/// Check that a SL (Signed Long) Value with VM > 1 (Multi Value) works
void ObjectAndElementTest::Test_MultiValueFL ()
{
	TestMultiValue(float_values, sizeof(float_values)/sizeof(float_values[0]), rzdcxLib::GraphicData);
}

void ObjectAndElementTest::Test_SingleValueFD ()
{
	TestSingleValue((double)-1.45E2, rzdcxLib::TableOfYBreakPoints);
}

double double_values[] = { 1.2, 3.4, 5.6, 7.8, 9. };
void ObjectAndElementTest::Test_MultiValueFD ()
{
	TestMultiValue(double_values, sizeof(double_values)/sizeof(double_values[0]), rzdcxLib::TableOfYBreakPoints);
}

template<typename T>
void TestSetFromString(T* values, unsigned long numValues, long tag)
{
	ostringstream stream;
	if (numValues > 0)
	{
		stream << values[0];
		for (unsigned long i = 1; i < numValues; i++)
		{
			stream << "\\" << values[i];
		}
	}
	
	IDCXOBJPtr o(__uuidof(DCXOBJ));
	IDCXELMPtr in = o->createElement(tag);
	in->Value = stream.str().c_str();
	o->insertElement(in);

	IDCXELMPtr out = o->getElementByTag(tag);

	CPPUNIT_ASSERT_EQUAL((T)out->Value, (T)values[0]);

	CPPUNIT_ASSERT_EQUAL((long)numValues, out->ValueMultiplicity);

	_variant_t v = out->ValueArray;
	SAFEARRAY *pSafeArray = v.parray;
	T* dwArray = NULL;
	SafeArrayAccessData(pSafeArray, (void**)&dwArray);
	for (unsigned long i=0; i<numValues; i++)
	{
		CPPUNIT_ASSERT_EQUAL((T)dwArray[i], values[i]);
	}
	SafeArrayUnaccessData(pSafeArray);
}

void ObjectAndElementTest::Test_SetFromStringSL ()
{
	TestSetFromString(long_values, sizeof(long_values)/sizeof(long_values[0]), rzdcxLib::DisplayedAreaTopLeftHandCorner);
}

void ObjectAndElementTest::Test_SetFromStringFL ()
{
	TestSetFromString(float_values, sizeof(float_values)/sizeof(float_values[0]), rzdcxLib::RecommendedDisplayFrameRateInFloat);
}

void ObjectAndElementTest::Test_SetFromStringFD ()
{
	TestSetFromString(double_values, sizeof(double_values)/sizeof(double_values[0]), rzdcxLib::TableOfYBreakPoints);
}

void ObjectAndElementTest::Test_SetSomeWiredString ()
{
	IDCXELMPtr e(__uuidof(DCXELM));
	e->Init(PatientsName);
	char someWiredString[] = "lkjhasdfalkklads\0 123";
	e->Length = sizeof(someWiredString);
	e->CStringPtr = (int)someWiredString;
	IDCXOBJPtr o(__uuidof(DCXOBJ));
	o->insertElement(e);
	o->saveFile("Test_SetSomeWiredString.dcm");
}