﻿using System;
using System.Collections.Generic;
using System.Text;
using rzdcxLib;
using System.IO;

namespace rzdcxNUnit
{
    class CommonTestUtilities
    {
        public static string TestPatientName
        {
            get { return "John^Doe"; }
        }

        public static string TestPatientID
        {
            get { return "123765"; }
        }

        public static string TestStudyInstanceUID
        {
            get { return "123765.1"; }
        }

        public static string TestSeriesInstanceUID
        {
            get { return "123765.1.1"; }
        }

        public static unsafe void CreateDummyImages(String path)
        {
            CreateDummyImages(path, 4, 2);
        }

        public static unsafe void CreateDummyImages(String path, int numSeries, int numImagesPerSeries)
        {
            const int ROWS = 64;
            const int COLUMNS = 64;
            const int SAMPLES_PER_PIXEL = 1;
            const string PHOTOMETRIC_INTERPRETATION = "MONOCHROME2";
            const int BITS_ALLOCATED = 16;
            const int BITS_STORED = 12;
            const int RESCALE_INTERCEPT = 0;

            DCXOBJ obj = new DCXOBJ();

            /// Create an element pointer to place in the object for every tag
            DCXELM el = new DCXELM();

            /// Set Hebrew Character Set
            el.Init((int)DICOM_TAGS_ENUM.SpecificCharacterSet);
            el.Value = "ISO_IR 192";
            /// insert the element to the object
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.Rows);
            el.Value = ROWS;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.Columns);
            el.Value = COLUMNS;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.SamplesPerPixel);
            el.Value = SAMPLES_PER_PIXEL;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.PhotometricInterpretation);
            el.Value = PHOTOMETRIC_INTERPRETATION;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.BitsAllocated);
            el.Value = BITS_ALLOCATED;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.BitsStored);
            el.Value = BITS_STORED;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.HighBit);
            el.Value = BITS_STORED - 1;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.PixelRepresentation);
            el.Value = 0;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.WindowCenter);
            el.Value = (int)(1 << (BITS_STORED - 1));
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.WindowWidth);
            el.Value = (int)(1 << BITS_STORED);
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.RescaleIntercept);
            el.Value = (short)RESCALE_INTERCEPT;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.RescaleSlope);
            el.Value = 1;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.GraphicData);
            el.Value = "456\\8934\\39843\\223\\332\\231\\100\\200\\300\\400";
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.PixelData);
            el.Length = ROWS * COLUMNS * SAMPLES_PER_PIXEL;
            el.ValueRepresentation = VR_CODE.VR_CODE_OW;

            ushort[] pixels = new ushort[ROWS * COLUMNS];
            for (int y = 0; y < ROWS; y++)
            {
                for (int x = 0; x < COLUMNS; x++)
                {
                    int i = x + COLUMNS * y;
                    pixels[i] = (ushort)(((i) % (1 << BITS_STORED)) - RESCALE_INTERCEPT);
                }
            }
            fixed (ushort* p = pixels)
            {
                UIntPtr p1 = (UIntPtr)p;
                el.Value = p1;
            }

            obj.insertElement(el);

            // Set identifying elements
            el.Init((int)DICOM_TAGS_ENUM.PatientsName);
            el.Value = CommonTestUtilities.TestPatientName;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.patientID);
            el.Value = TestPatientID;
            obj.insertElement(el);

            String study_uid = "123765.1";
            el.Init((int)DICOM_TAGS_ENUM.studyInstanceUID);
            el.Value = CommonTestUtilities.TestStudyInstanceUID;
            obj.insertElement(el);

            el.Init((int)DICOM_TAGS_ENUM.StudyID);
            el.Value = 1;
            obj.insertElement(el);


            el.Init((int)DICOM_TAGS_ENUM.sopClassUid);
            el.Value = "1.2.840.10008.5.1.4.1.1.7"; // Secondary Capture
            obj.insertElement(el);


            for (int seriesid = 1; seriesid <= numSeries; seriesid++)
            {
                Directory.CreateDirectory(path + "\\SER" + seriesid);

                String series_uid = study_uid + "." + seriesid;
                el.Init((int)DICOM_TAGS_ENUM.seriesInstanceUID);
                el.Value = series_uid;
                obj.insertElement(el);

                el.Init((int)DICOM_TAGS_ENUM.SeriesNumber);
                el.Value = seriesid;
                obj.insertElement(el);

                for (int instanceid = 1; instanceid <= numImagesPerSeries; instanceid++)
                {
                    el.Init((int)DICOM_TAGS_ENUM.sopInstanceUID);
                    el.Value = series_uid + "." + instanceid;
                    obj.insertElement(el);

                    el.Init((int)DICOM_TAGS_ENUM.InstanceNumber);
                    el.Value = instanceid;
                    obj.insertElement(el);

                    /// Save it
                    obj.saveFile(path + "\\SER" + seriesid + "\\IMG" + instanceid);
                }
            }
        }
    }
}
