﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using rzdcxLib;

namespace rzdcxNUnit
{
    [TestFixture]
    class Test_Misc
    {
        [Test]
        public void TestCreateUID()
        {
            DCXUID uid = new DCXUID();
            string study_uid = uid.CreateUID(UID_TYPE.UID_TYPE_STUDY);
            string series_uid = uid.CreateUID(UID_TYPE.UID_TYPE_SERIES);
            string instance_uid = uid.CreateUID(UID_TYPE.UID_TYPE_INSTANCE);
            string mpps_uid = uid.CreateUID(UID_TYPE.UID_TYPE_MPPS);
            string scm_uid = uid.CreateUID(UID_TYPE.UID_TYPE_SCM_TRANSACTION);
            string other_uid = uid.CreateUID(UID_TYPE.UID_TYPE_OTHER);
            
            uid.SetRoot("1.2.3");
            
            study_uid = uid.CreateUID(UID_TYPE.UID_TYPE_STUDY);
            Assert.That(study_uid.Substring(0, 5) == "1.2.3");
            
            series_uid = uid.CreateUID(UID_TYPE.UID_TYPE_SERIES);
            Assert.That(series_uid.Substring(0, 5) == "1.2.3"); 
            
            instance_uid = uid.CreateUID(UID_TYPE.UID_TYPE_INSTANCE);
            Assert.That(instance_uid.Substring(0, 5) == "1.2.3");
            
            mpps_uid = uid.CreateUID(UID_TYPE.UID_TYPE_MPPS);
            Assert.That(mpps_uid.Substring(0, 5) == "1.2.3"); 
            
            scm_uid = uid.CreateUID(UID_TYPE.UID_TYPE_SCM_TRANSACTION);
            Assert.That(scm_uid.Substring(0, 5) == "1.2.3");

            other_uid = uid.CreateUID(UID_TYPE.UID_TYPE_OTHER);
            Assert.That(other_uid.Substring(0, 5) == "1.2.3");
        }

        private bool NewInstanceUIDChecker(NEW_INSTANCE_CREATION_POLICY policy)
        {
            /// Set never create new UID
            DCXAPP app = new DCXAPP();
            app.NewInstanceUIDPolicy = policy;

            /// Create a file and save it, keep the uid
            CommonTestUtilities.CreateDummyImages("NEWINSTUID", 1, 1);
            string filename1 = "NEWINSTUID" + "\\SER1\\IMG1";
            DCXOBJ o = new DCXOBJ();
            o.openFile(filename1);

            /// Compress it and save it again
            o.EncodeJpeg(50);
            string filename2 = filename1 + "JPG";
            o.saveFile(filename2);

            DCXOBJ o1 = new DCXOBJ();
            o1.openFile(filename1);
            DCXOBJ o2 = new DCXOBJ();
            o2.openFile(filename2);

            DCXELM uid1 = o1.getElementByTag((int)DICOM_TAGS_ENUM.sopInstanceUID);
            DCXELM uid2 = o2.getElementByTag((int)DICOM_TAGS_ENUM.sopInstanceUID);
            return uid1.Value.ToString() == uid2.Value.ToString();
        }

        [Test]
        public void TestNewInstanceUIDPolicy()
        {
            Assert.True(NewInstanceUIDChecker(NEW_INSTANCE_CREATION_POLICY.CNU_NEVER));
            Assert.False(NewInstanceUIDChecker(NEW_INSTANCE_CREATION_POLICY.CNU_ALWAYS));
            Assert.False(NewInstanceUIDChecker(NEW_INSTANCE_CREATION_POLICY.CNU_ONLOSSYCOMPRESSION));
        }
    }
}
