﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using System.IO;
using rzdcxLib;

namespace rzdcxNUnit
{
    [TestFixture]
    public class Test_ObjectAndElement
    {
        public Test_ObjectAndElement()
        {
            //DCXAPP app = new DCXAPP();
         //   app.StartLogging("rzdcx.log");
           // app.LogLevel = LOG_LEVEL.LOG_LEVEL_ERROR;
        }
        /// <summary>
        /// Test element value of type PN using the Patient Name tag
        /// </summary>
        [Test]
        public void ElementSetGetValue_PN_PatientName()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.PatientsName);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_PN, "Patient's Name VR should be PN");

            e.Value = "DOE^JOHN";

            Assert.AreEqual("DOE^JOHN", e.Value, "PN Value set and value get does not match");
        }

        [Test]
        public void ElementSetGetValue_US_Rows()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.Rows);
            
            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_US, "VR should be US");

            e.Value = 512;

            Assert.AreEqual(512, e.Value, "US Value set and get does not match");
        }

        [Test]
        public void ElementSetGetValue_SS_TIDOffset()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.TIDOffset);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_SS, "VR should be SS");

            e.Value = -100;

            Assert.AreEqual(-100, e.Value, "SS Value set and get does not match");
        }

        [Test]
        public void ElementSetGetValue_StringToFloat()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.MaskSubPixelShift);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_FL, "VR should be FL");

            e.Value = "-512E-2";

            Assert.AreEqual((float)-512E-2, (float)e.Value, "FL Value set from string and get does not match");
        }

        [Test]
        public void ElementSetGetValue_AttributeTag()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.DimensionIndexPointer);
            
            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_AT, "VR should be AT");

            e.Value = 0x12345678;

            Assert.AreEqual(0x12345678, e.Value, "AT Values from set and get does not match");
        }

        [Test]
        public void ElementSetGetValue_FD()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.RealWorldValueLUTData);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_FD, "VR should be FD");

            e.Value = -123456789.994499449944;

            Assert.AreEqual(-123456789.994499449944, e.Value, "FD Value set from string and get does not match");
        }

        [Test]
        public unsafe void ElementSetGetMultival_FD()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.ImagePositionVolume);
            e.Length = 3 * sizeof(double);

            double[] values = new double[3];

            values[0] = 1.23;
            values[1] = 4.56;
            values[2] = -0.98;

            fixed (double* p = values)
            {
                UIntPtr p1 = (UIntPtr)p;
                e.Value = p1;
            }

            double[] res = (e.ValueArray as double[]);
            Assert.AreEqual(res[0], 1.23);
            Assert.AreEqual(res[1], 4.56);
            Assert.AreEqual(res[2], -0.98);
        }


        [Test]
        public void ElementSetGetValue_SL()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.SelectorSLValue);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_SL, "Incorrect VR");

            e.Value = -65537;

            Assert.AreEqual(-65537, e.Value, "Get and Set Values does not match");
        }

        [Test]
        public void ElementSetGetValue_SS()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.TagAngleSecondAxis);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_SS, "Incorrect VR");

            e.Value = -32767;

            Assert.AreEqual(-32767, e.Value, "Get and Set Values does not match");
        }

        [Test]
        public void ElementSetGetValue_UL()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.SpectroscopyAcquisitionPhaseColumns);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_UL, "Incorrect VR");

            e.Value = 4194961296;

            Assert.AreEqual(4194961296, e.Value, "Get and Set Values does not match");
        }

        [Test]
        public void ElementSetGetValue_US()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.RFEchoTrainLength);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_US, "Incorrect VR");

            e.Value = 12345;

            Assert.AreEqual(12345, e.Value, "Get and Set Values does not match");
        }

        [Test]
        public unsafe void ElementSetGetValue_PixelData()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.PixelData);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_OW, "Incorrect VR");

            e.Length = 8;

            ushort[] pixels = new ushort[8];

            for (ushort i = 0; i < 8; i++)
                pixels[i] = i;

            fixed (ushort* p = pixels)
            {
                UIntPtr p1 = (UIntPtr)p;
                e.Value = p1;
            }         
        }

        /// <summary>
        /// Setting sequence with empty iterator never returns? Bug reported by Greg (STI-HAWAII)
        /// </summary>
        [Test]
        public void ElementSetEmptySequence()
        {
            DCXOBJIterator i1 = new DCXOBJIterator();
            DCXELM e = new DCXELM();
            e.Init((int)DICOM_TAGS_ENUM.ReferencedStudySequence);
            e.Value = i1;

            DCXOBJIterator i2 = e.Value as DCXOBJIterator;
            Assert.That(i2 == null || i2.AtEnd());
        }

        [Test]
        public void ElementSetGetMultival_StringToFloat()
        {
            String ss = "936\\773\\936\\773\\945\\763\\1272\\650\\1753\\768\\1839\\1145\\1843\\1286\\1866\\1399\\1880\\1463\\1912\\1540\\1912\\1554";
            DCXELM e = new DCXELM();
            e.Init(0x00700022);
            e.Value = ss;
            float[] f = (e.ValueArray as float[]);
            Assert.That(f.Length == 22);
            Assert.That(f[0] == 936);
            Assert.That(f[f.Length - 1] == 1554);
        }

        [Test]
        public void ElementSetGetValue_ImageType()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.ImageType);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_CS, "Image Type VR should be CS");

            e.Value = "A\\B\\C\\D";

            int vm = e.ValueMultiplicity;

            Assert.AreEqual(vm, 4);

            Assert.AreEqual("A\\B\\C\\D", e.Value, "Image Type (CS) Value set and value get does not match");
        }
        
        [Test]
        public void ElementSetGetValue_StudyTime()
        {
            rzdcxLib.DCXELM e = new DCXELM();
            e.Init((int)rzdcxLib.DICOM_TAGS_ENUM.StudyTime);

            Assert.AreEqual(e.ValueRepresentation, rzdcxLib.VR_CODE.VR_CODE_TM, "Study Time VR should be TM");

            e.Value = "130459.123";

            Assert.AreEqual("13:04:59.123", e.Value, "Study Time (TM) Value set and value get does not match");
        }

        [Test]
        public void ValueArray()
        {
            DCXELM e1 = new DCXELM();
            e1.Init((int)DICOM_TAGS_ENUM.WindowCenter);
            e1.Value = "1\\2\\3\\4\\5";
            DCXOBJ o = new DCXOBJ();
            o.insertElement(e1);
            DCXELM e2 = o.getElementByTag((int)DICOM_TAGS_ENUM.WindowCenter);
            Array a = e2.ValueArray as Array;
            Assert.AreEqual(a.Length, 5);
            Assert.AreEqual("1", a.GetValue(0));
            Assert.AreEqual("2", a.GetValue(1));
            Assert.AreEqual("3", a.GetValue(2));
            Assert.AreEqual("4", a.GetValue(3));
            Assert.AreEqual("5", a.GetValue(4));
        }

        [Test]
        [Category("FileMetaInfo")]
        [Description("Get File Meta Info of a newly created object should return a NULL object")]
        public void GetMetaInfoOfEmptyObject()
        {
            DCXOBJ obj = new DCXOBJ();
            DCXOBJ metaInfo = obj.FileMetaInfo;
            Assert.IsNull(metaInfo);
        }

        private DCXOBJ CreateTestFileAndOpenIt(out string filename)
        {
            String fullpath = "METAINFO";
            Directory.CreateDirectory(fullpath);
            CommonTestUtilities.CreateDummyImages(fullpath, 1, 1);
            DCXOBJ obj = new DCXOBJ();
            filename = fullpath + "\\SER1\\IMG1";
            obj.openFile(filename);
            return obj;
        }

        [Test]
        [Category("FileMetaInfo")]
        [Description("Set and Get File Meta Info")]
        public void SetAndGetFileMetaInfo()
        {
            string filename;
            DCXOBJ obj = CreateTestFileAndOpenIt(out filename);
            Assert.IsNotNull(obj);
            
            // Read the meta info
            DCXOBJ metaInfo = obj.FileMetaInfo;
            Assert.IsNotNull(metaInfo);

            // Check that the implementation UID and Version name are of RZDCX
            DCXELM implementationUID = metaInfo.getElementByTag((int)DICOM_TAGS_ENUM.ImplementationClassUID);
            Assert.That(implementationUID.Value.ToString().StartsWith("2.16.124.113543.6021"));

            DCXELM versionName = metaInfo.getElementByTag((int)DICOM_TAGS_ENUM.ImplementationVersionName);
            Assert.That(versionName.Value.ToString().StartsWith("RZDCX_"));

            // Now change the meta info and save a copy of the file
            implementationUID.Value = "1.2.3";
            metaInfo.insertElement(implementationUID);

            versionName.Value = "NUNIT 2.5.4";
            metaInfo.insertElement(versionName);
            
            obj.FileMetaInfo = metaInfo;
            obj.saveFile(filename + "_NEW");

            // Now read it and check that the values has changed

            DCXOBJ newObj = new DCXOBJ();
            newObj.openFile(filename + "_NEW");
            DCXOBJ newMetaInfo = newObj.FileMetaInfo;
            Assert.IsNotNull(metaInfo);

            // Check that the implementation UID and Version name are of RZDCX
            DCXELM newImplementationUID = newMetaInfo.getElementByTag((int)DICOM_TAGS_ENUM.ImplementationClassUID);
            Assert.That(newImplementationUID.Value.ToString() == "1.2.3");

            DCXELM newVersionName = newMetaInfo.getElementByTag((int)DICOM_TAGS_ENUM.ImplementationVersionName);
            Assert.That(newVersionName.Value.ToString() == "NUNIT 2.5.4");

            // Delete working files
            File.Delete(filename);
            File.Delete(filename + "_NEW");
        }

        // Currently there's no validation on that
        //[Test]
        //[Category("FileMetaInfo")]
        //[Description("Set meta info with different transfer syntax should change encoding or return error")]
        //public void SetFileMetaInfoChangeTransferSyntax()
        //{
        //    string filename;
        //    DCXOBJ obj = CreateTestFileAndOpenIt(out filename);
        //    DCXOBJ meta = new DCXOBJ();
        //    DCXELM ts = new DCXELM();
        //    ts.Init((int)DICOM_TAGS_ENUM.TransferSyntaxUID);
        //    ts.Value = "1.2.840.10008.1.2.4.50";
        //    meta.insertElement(ts);
        //    obj.FileMetaInfo = meta;
        //    obj.saveFile(filename + "_NEW");

        //    DCXOBJ newObj = new DCXOBJ();
        //    newObj.openFile(filename + "_NEW");
        //    meta = newObj.FileMetaInfo;
        //    DCXELM newTs = meta.getElementByTag((int)DICOM_TAGS_ENUM.TransferSyntaxUID);
        //    Assert.AreEqual(newTs.Value.ToString(), "1.2.840.10008.1.2.4.50");
        //}

        [Test]
        [Category("FileMetaInfo")]
        [Description("New objects don't have Meta Header")]
        public void NoMetaHeaderForNewObject()
        {
            DCXOBJ obj = new DCXOBJ();
            DCXOBJ meta = obj.FileMetaInfo;
            Assert.IsNull(meta);
            
        }

        [Test]
        [Category("FileMetaInfo")]
        [Description("If only part of the elements are set add the missing")]
        public void FillMissingElements()
        {
            DCXOBJ obj = new DCXOBJ();
            DCXOBJ meta = obj.FileMetaInfo;
            Assert.IsNull(meta);
            DCXELM e = new DCXELM();
            e.Init((int)DICOM_TAGS_ENUM.PatientsName);
            e.Value = "DOE^JOHN^MR";
            obj.insertElement(e);
            meta = new DCXOBJ();
            e.Init((int)DICOM_TAGS_ENUM.ImplementationClassUID);
            e.Value = "1.2.3";
            meta.insertElement(e);
            obj.FileMetaInfo = meta;
            obj.saveFile("dummy");
            DCXOBJ obj2 = new DCXOBJ();
            obj2.openFile("dummy");
            DCXOBJ meta2 = obj2.FileMetaInfo;
            e = meta2.getElementByTag((int)DICOM_TAGS_ENUM.ImplementationClassUID);
            Assert.AreEqual(e.Value.ToString(), "1.2.3");
            e = meta2.getElementByTag((int)DICOM_TAGS_ENUM.ImplementationVersionName);
            Assert.That(e.Value.ToString().StartsWith("RZDCX_"));
            File.Delete("dummy");
        }

        [Test]
        [Category("FileMetaInfo")]
        [Description("Test the utility function that checks existance of a meta info")]
        public void FileMetaInfoCheckExists()
        {
            DCXOBJ obj = new DCXOBJ();
            Assert.False(obj.FileMetaInfoExists);
            DCXOBJ meta = new DCXOBJ();
            obj.FileMetaInfo = meta;
            Assert.True(obj.FileMetaInfoExists);
            obj.saveFile("dummy");

            DCXOBJ file = new DCXOBJ();
            file.openFile("dummy");
            Assert.True(file.FileMetaInfoExists);
        }

        [Test]
        [Category("FileMetaInfo")]
        [Description("Change transfer syntax and set meta info")]
        public void ChangeTransferSyntaxAndMetaInfo()
        {
            /// Create a test file
            string filename;
            DCXOBJ obj = CreateTestFileAndOpenIt(out filename);
            Assert.IsNotNull(obj);

            /// Change transfer syntax to JPEG Lossy
            obj.TransferSyntax = TRANSFER_SYNTAX.TS_LOSSLESS_JPEG_DEFAULT;

            obj.saveFile(filename + "_JPEG");

            /// Now change meta info
            DCXOBJ meta = new DCXOBJ();

            DCXELM implementationUID = new DCXELM();
            implementationUID.Init((int)DICOM_TAGS_ENUM.ImplementationClassUID);
            implementationUID.Value = "1.2.3";
            meta.insertElement(implementationUID);

            DCXELM versionName = new DCXELM();
            versionName.Init((int)DICOM_TAGS_ENUM.ImplementationVersionName);
            versionName.Value = "NUNIT 2.5.4";
            meta.insertElement(versionName);

            DCXELM transferSyntax = new DCXELM();
            transferSyntax.Init((int)DICOM_TAGS_ENUM.TransferSyntaxUID);
            transferSyntax.Value = "1.2.840.10008.1.2.4.70";
            
            obj.FileMetaInfo = meta;
            obj.saveFile(filename + "_JPEG_META");

            DCXOBJ jpegObj = new DCXOBJ();
            jpegObj.openFile(filename + "_JPEG");
            string tr1 = jpegObj.FileMetaInfo.getElementByTag((int)DICOM_TAGS_ENUM.TransferSyntaxUID).Value.ToString();

            DCXOBJ jpegMetaObj = new DCXOBJ();
            jpegMetaObj.openFile(filename + "_JPEG_META");
            string tr2 = jpegMetaObj.FileMetaInfo.getElementByTag((int)DICOM_TAGS_ENUM.TransferSyntaxUID).Value.ToString();

            Assert.AreEqual(tr1, tr2);
        }

        [Test]
        public unsafe void SetValueFromFile()
        {
            FileStream stream = File.Create("SetFromFileData.dat");
            BinaryWriter writer = new BinaryWriter(stream);
            byte[] somedata = new byte[100];

            writer.Write(somedata, 0, 100);
            for (int i = 0; i < 1024 * 1024 * 30; i++) // 1 MB data
            {
                byte val = (byte)(i % 256);
                writer.Write(val);
            }
            writer.Write(somedata, 0, 100); 
            writer.Close();
          
            DCXOBJ o = new DCXOBJ();
            DCXELM e = new DCXELM();
            e.Init((int)DICOM_TAGS_ENUM.PixelData);
            e.ValueRepresentation = VR_CODE.VR_CODE_OB;
            e.SetValueFromFile("SetFromFileData.dat", 100, 1024 * 1024 * 30);
            o.insertElement(e);
            o.saveFile("SetFromFileData.dcm");

            DCXOBJ o2 = new DCXOBJ();
            o2.openFile("SetFromFileData.dcm");
            DCXELM e2 = o2.getElementByTag((int)DICOM_TAGS_ENUM.PixelData);
            UInt32 p = (UInt32)e2.Value;
            byte* bytes = (byte*)p;
            Assert.That(bytes[0] == 0);
            Assert.That(bytes[1] == 1);
            Assert.That(bytes[255] == 255);
            Assert.That(bytes[1024 * 256 + 0] == 0);
            Assert.That(bytes[1024 * 256 + 1] == 1);
            Assert.That(bytes[1024 * 256 + 255] == 255);
        }

        [Test]
        public unsafe void ElementSetGetRawData()
        {
                                     //              1
            DCXELM e = new DCXELM(); //     123456789012
            e.Init((int)DICOM_TAGS_ENUM.ReferringPhysicianName);
            string someInStr = "ChineezeStuf";
            int inLen = someInStr.Length * sizeof(char);
            fixed (char* p = someInStr.ToCharArray())
            {
                e.SetRawData((int)p, inLen);
            }

            int outLen;
            string someOutStr;
            int q;
            e.GetRawData(out q, out outLen);
            someOutStr = new string((char*)q, 0, outLen/sizeof(char));
            
            Assert.That(someInStr == someOutStr);
            Assert.That(inLen == outLen);

            DCXOBJ o = new DCXOBJ();
            o.insertElement(e);
            o.saveFile("chineeze.dcm");
            DCXOBJ o1 = new DCXOBJ();
            o1.openFile("chineeze.dcm");
            DCXELM e1 = o1.getElementByTag((int)DICOM_TAGS_ENUM.ReferringPhysicianName);
            int outLen1;
            string someOutStr1;
            int r;
            e.GetRawData(out r, out outLen1);
            someOutStr1 = new string((char*)r, 0, outLen1 / sizeof(char));
        
            Assert.That(someInStr == someOutStr1);
            Assert.That(inLen == outLen1);

            File.Delete("chineeze.dcm");
        }

        [Test]
        [Category("Attended")]
        public void ElementNotFoundError()
        {
            DCXOBJ o = new DCXOBJ();
            DCXELM e = o.getElementByTag((int)DICOM_TAGS_ENUM.StudyDate);
        }


   }
}
