﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using rzdcxLib;
using System.Threading;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace rzdcxNUnit
{
    /// <summary>
    /// Test RZDCX Requester Class
    /// </summary>
    [TestFixture]
    [Category("PRINT SCU")]
    public class Test_Print
    {
        static string MyAETitle = System.Environment.GetEnvironmentVariable("COMPUTERNAME");

        static string PRINT_AE = Properties.Settings.Default.PRINT_SERVER_AE;
        static string PRINT_Host = Properties.Settings.Default.PRINT_SERVER_HOST;
        static ushort PRINT_port = Properties.Settings.Default.PRINT_SERVER_PORT;


        [Test]
        public void GetPrinterInfo()
        {
            DCXREQ r = new DCXREQ();
            DCXOBJ printerInfo = r.GetPrinterInfo(MyAETitle, PRINT_AE, PRINT_Host, PRINT_port);
            Assert.AreEqual("NORMAL", printerInfo.getElementByTag((int)DICOM_TAGS_ENUM.PrinterStatus).Value);
            Assert.AreEqual("NORMAL", printerInfo.getElementByTag((int)DICOM_TAGS_ENUM.PrinterStatusInfo).Value);
        }

        [Test]
        public void GetPrinterInfoInGrayscaleSession()
        {
            DCXREQ r = new DCXREQ();
            DCXPrintSession session = r.StartGrayscalePrintSession(MyAETitle, PRINT_AE, PRINT_Host, PRINT_port);
            DCXOBJ printerInfo = session.GetPrinterInfo();
            Assert.AreEqual("NORMAL", printerInfo.getElementByTag((int)DICOM_TAGS_ENUM.PrinterStatus).Value);
            Assert.AreEqual("NORMAL", printerInfo.getElementByTag((int)DICOM_TAGS_ENUM.PrinterStatusInfo).Value);
        }

        [Test]
        public void GetPrinterInfoInColorSession()
        {
            DCXREQ r = new DCXREQ();
            DCXPrintSession session = r.StartColorPrintSession(MyAETitle, PRINT_AE, PRINT_Host, PRINT_port);
            DCXOBJ printerInfo = session.GetPrinterInfo();
            Assert.AreEqual("NORMAL", printerInfo.getElementByTag((int)DICOM_TAGS_ENUM.PrinterStatus).Value);
            Assert.AreEqual("NORMAL", printerInfo.getElementByTag((int)DICOM_TAGS_ENUM.PrinterStatusInfo).Value);
        }

        [Test]
        public unsafe void PrintGrayscale()
        {
            DCXREQ requester = new DCXREQ();
            DCXPrintSession session = requester.StartGrayscalePrintSession(MyAETitle, PRINT_AE, PRINT_Host, PRINT_port);
            DCXPrintPage page = new DCXPrintPage();
            page.Orientation = PRINT_PAGE_ORIENTATION.PPO_PORTRAIT;
            page.StandardLayout = @"STANDARD\1,1";
            session.CreatePage(page);
            DCXPrintImageBox imageBox = page.GetImageBox(1);
            imageBox.BitsAllocated = IMAGE_BITS_ALLOCATED.IBA_16;
            imageBox.BitsStored = IMAGE_BITS_STORED.IBS_12;
            imageBox.HighBit = IMAGE_HIGH_BIT.IHB_11;
            imageBox.Rows = 128;
            imageBox.Columns = 128;
            imageBox.SamplesPerPixel = IMAGE_SAMPLES_PER_PIXEL.ISPP_1;
            imageBox.PixelRepresentation = IMAGE_PIXEL_REPRESENTATION.IPR_UNSIGNED;
            imageBox.PhotometricInterpretation = IMAGE_PHOTOMETRIC_INTERPRETATION.IPI_MONOCHROME2;
            fixed (ushort* p = new ushort[128 * 128])
            {
                imageBox.PixelDataPtr = (int)p;
            }
            page.SetImageBox(1, imageBox);
            page.Print();
            session.DeletePage(page);
            requester.EndPrintSession();
        }

        [Test]
        public unsafe void GrayscalePrintSessionExtras()
        {
            DCXOBJ sessionAttributes = new DCXOBJ();
            DCXELM e = new DCXELM();
            e.Init((int)DICOM_TAGS_ENUM.NumberOfCopies);
            e.Value = 3;
            sessionAttributes.insertElement(e);
            DCXREQ requester = new DCXREQ();
            DCXPrintSession session = requester.StartGrayscalePrintSessionEx(MyAETitle, PRINT_AE, PRINT_Host, PRINT_port, sessionAttributes);
            requester.EndPrintSession();
        }

        [Test]
        public unsafe void PrintColor()
        {
            DCXREQ requester = new DCXREQ();
            DCXPrintSession session = requester.StartColorPrintSession(MyAETitle, PRINT_AE, PRINT_Host, PRINT_port);
            DCXPrintPage page = new DCXPrintPage();
            page.Orientation = PRINT_PAGE_ORIENTATION.PPO_PORTRAIT;
            page.StandardLayout = @"STANDARD\1,1";
            session.CreatePage(page);
            DCXPrintImageBox imageBox = page.GetImageBox(1);
            imageBox.BitsAllocated = IMAGE_BITS_ALLOCATED.IBA_8;
            imageBox.BitsStored = IMAGE_BITS_STORED.IBS_8;
            imageBox.HighBit = IMAGE_HIGH_BIT.IHB_7;
            imageBox.Rows = 128;
            imageBox.Columns = 128;
            imageBox.SamplesPerPixel = IMAGE_SAMPLES_PER_PIXEL.ISPP_3;
            imageBox.PixelRepresentation = IMAGE_PIXEL_REPRESENTATION.IPR_UNSIGNED;
            imageBox.PhotometricInterpretation = IMAGE_PHOTOMETRIC_INTERPRETATION.IPI_RGB;
            imageBox.PlanarConfiguration = IMAGE_PLANAR_CONFIGURATION.IPC_INTERLEAVE;
            fixed (char* p = new char[128 * 128 * 3])
            {
                imageBox.PixelDataPtr = (int)p;
            }
            page.SetImageBox(1, imageBox);
            page.Print();
            // This can be done only with printers that send N-EVENT-REPORT
            //DCXOBJ printerInfo = session.WaitForPrinterInfo(180);
            session.DeletePage(page);
            requester.EndPrintSession();
        }

        [Test]
        public unsafe void ColorPrintSessionExtras()
        {
            DCXOBJ sessionAttributes = new DCXOBJ();
            DCXELM e = new DCXELM();
            e.Init((int)DICOM_TAGS_ENUM.NumberOfCopies);
            e.Value = 3;
            sessionAttributes.insertElement(e);
            DCXREQ requester = new DCXREQ();
            DCXPrintSession session = requester.StartColorPrintSessionEx(MyAETitle, PRINT_AE, PRINT_Host, PRINT_port, sessionAttributes);
            requester.EndPrintSession();
        }

        [Test]
        [Category("Attended")]
        [Description("This test requires a Print SCP that supports N-EVENT-REPORT for Printer SOP Class")]
        public void GrayscalePrinterInfoNEventReport()
        {
            DCXREQ req = new DCXREQ();
            DCXPrintSession session = req.StartGrayscalePrintSession(MyAETitle, PRINT_AE, PRINT_Host, PRINT_port);
            DCXOBJ printerInfo = session.WaitForPrinterInfo(180);
            DCXELM status = printerInfo.getElementByTag((int)DICOM_TAGS_ENUM.PrinterStatus);
            Assert.AreEqual(status.Value, "NORMAL");
            Marshal.ReleaseComObject(status);
            Marshal.ReleaseComObject(printerInfo);
            Marshal.ReleaseComObject(session);
            Marshal.ReleaseComObject(req);
        }
        [Test]
        [Category("Attended")]
        [Description("This test requires a Print SCP that supports N-EVENT-REPORT for Printer SOP Class")]
        public void ColorPrinterInfoNEventReport()
        {
            DCXREQ req = new DCXREQ();
            DCXPrintSession session = req.StartColorPrintSession(MyAETitle, PRINT_AE, PRINT_Host, PRINT_port);
            DCXOBJ printerInfo = session.WaitForPrinterInfo(30);
            DCXELM status = printerInfo.getElementByTag((int)DICOM_TAGS_ENUM.PrinterStatus);
            Assert.AreEqual(status.Value, "NORMAL");
        }
    }
}
