﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using rzdcxLib;
using System.Threading;
using System.Diagnostics;
using System.Runtime.InteropServices;
using System.IO;

namespace rzdcxNUnit
{
    /// <summary>
    /// Test the RZDCX Requester Class
    /// </summary>
    [TestFixture]
    public class Test_Requester
    {
        static string MyAETitle = System.Environment.GetEnvironmentVariable("COMPUTERNAME");

        static string IS_AE = Properties.Settings.Default.IMAGE_SERVER_AE;
        static string IS_Host = Properties.Settings.Default.IMAGE_SERVER_HOST;
        static ushort IS_port = Properties.Settings.Default.IMAGE_SERVER_PORT;

        static string RIS_AE = Properties.Settings.Default.RIS_SERVER_AE;
        static string RIS_Host = Properties.Settings.Default.RIS_SERVER_HOST;
        static ushort RIS_port = Properties.Settings.Default.RIS_SERVER_PORT;

        public Test_Requester()
        {
            //SendTestStudy();
         //   DCXAPP app = new DCXAPP();
           // app.StartLogging("rzdcx.log");
         //   app.LogLevel = LOG_LEVEL.LOG_LEVEL_ERROR;
        }

        [Test]
        public void SendTestStudy()
        {
            String fullpath = "REQUESTERTEST";
            Directory.CreateDirectory(fullpath);
            CommonTestUtilities.CreateDummyImages(fullpath);
            DCXREQ r = new DCXREQ();
            string succeededInstances;
            string failedInstances;
            r.Send(MyAETitle, IS_AE, IS_Host, IS_port, fullpath + "\\SER1\\IMG1", out succeededInstances, out failedInstances);
            r.Send(MyAETitle, IS_AE, IS_Host, IS_port, fullpath + "\\SER1\\IMG2", out succeededInstances, out failedInstances);
            r.Send(MyAETitle, IS_AE, IS_Host, IS_port, fullpath + "\\SER2\\IMG1", out succeededInstances, out failedInstances);
            r.Send(MyAETitle, IS_AE, IS_Host, IS_port, fullpath + "\\SER2\\IMG2", out succeededInstances, out failedInstances);
            r.Send(MyAETitle, IS_AE, IS_Host, IS_port, fullpath + "\\SER3\\IMG1", out succeededInstances, out failedInstances);
            r.Send(MyAETitle, IS_AE, IS_Host, IS_port, fullpath + "\\SER3\\IMG2", out succeededInstances, out failedInstances);
            r.Send(MyAETitle, IS_AE, IS_Host, IS_port, fullpath + "\\SER4\\IMG1", out succeededInstances, out failedInstances);
            r.Send(MyAETitle, IS_AE, IS_Host, IS_port, fullpath + "\\SER4\\IMG2", out succeededInstances, out failedInstances);
            Directory.Delete(fullpath, true);
        }

        /// <summary>
        /// Test the C-ECHO DICOM Command
        /// </summary>
        [Test]
        public void Echo()
        {
            DCXREQ r = new DCXREQ();
            r.Echo(MyAETitle, IS_AE, IS_Host, IS_port);
        }

        class MoveTestAccepterHandler
        {
            public bool running = true;
            public bool gotfiles = false;

            public void Stop()
            {
                running = false;
            }

            public void Accept()
            {
                DCXACC a = new DCXACC();
                a.OnStoreDone += new IDCXACCEvents_OnStoreDoneEventHandler(OnStoreDone);
                a.OnConnection += new IDCXACCEvents_OnConnectionEventHandler(OnConnection);
                a.OnStoreSetup += new IDCXACCEvents_OnStoreSetupEventHandler(OnStoreSetup);
                int AcceptLoopCounter = 30;

                string MyAETitle = System.Environment.GetEnvironmentVariable("COMPUTERNAME");

                // Release the wait for this accepter to start
                lock (this)
                {
                    Monitor.Pulse(this);
                }

                /// Loop for at most AcceptTimout seconds and then 
                while (running && AcceptLoopCounter-- > 0)
                {
                    if (a.WaitForConnection(MyAETitle, 104, 5))
                    {
                        bool res;
                        do
                        {
                            if (a.WaitForCommand(5))
                            {
                                res = true;
                            }
                            else
                            {
                                res = false;
                            }
                        } while (res);
                    }
                }
            }

            int filecount=1;

            void OnStoreDone(string filename, bool storage_status, ref bool accept_storage)
            {
                accept_storage = true;
                File.Delete(filename);
            }

            void OnStoreSetup(ref string filename)
            {
                gotfiles = true;
                filename = "DCM" + filecount++;
            }

            void OnConnection(string calling_ae_title, string called_ae_title, string calling_host, ref bool accept_connection)
            {
                accept_connection = true;
            }
        }

        [Test]
        [Category("Query/Retrieve")]
        public void MovePatient()
        {
           MoveTestAccepterHandler a = new MoveTestAccepterHandler();
           Thread t = new Thread(new ThreadStart(a.Accept));
           t.Start();

            // Wait for the accepter to start
           lock (a)
           {
               Monitor.Wait(a);
           }
           try
           {
               DCXREQ r = new DCXREQ();
               r.MovePatient(MyAETitle, IS_AE, IS_Host, IS_port, MyAETitle, 
                   CommonTestUtilities.TestPatientName, CommonTestUtilities.TestPatientID);
           }
           finally
           {
               Assert.True(a.gotfiles);
               a.Stop();
               t.Join();
           }
        }

        [Test]
        [Category("Query/Retrieve")]
        public void MoveStudy()
        {
            MoveTestAccepterHandler a = new MoveTestAccepterHandler();
            Thread t = new Thread(new ThreadStart(a.Accept));
            t.Start();
            lock (a)
            {
                Monitor.Wait(a);
            }
            try
            {
                DCXREQ r = new DCXREQ();
                r.MoveStudy(MyAETitle, IS_AE, IS_Host, IS_port, MyAETitle, 
                    CommonTestUtilities.TestPatientID, CommonTestUtilities.TestStudyInstanceUID);
            }
            finally
            {
                Assert.True(a.gotfiles);
                a.Stop();
                t.Join();
            }
        }
        [Test]
        [Category("Query/Retrieve")]
        public void MoveSeries()
        {
            MoveTestAccepterHandler a = new MoveTestAccepterHandler();
            Thread t = new Thread(new ThreadStart(a.Accept));
            t.Start();
            lock (a)
            {
                Monitor.Wait(a);
            }
            try
            {
                DCXREQ r = new DCXREQ();
                r.MoveSeries(MyAETitle, IS_AE, IS_Host, IS_port, MyAETitle,
                    CommonTestUtilities.TestPatientID, CommonTestUtilities.TestStudyInstanceUID, CommonTestUtilities.TestSeriesInstanceUID);
            }
            finally
            {
                Assert.True(a.gotfiles);
                a.Stop();
                t.Join();
            }
		}

        class QueryHandler
        {
            private void OnQueryResponse(DCXOBJ obj)
            {
                string result = null;
                DCXELMIterator it = obj.iterator();
                while (it.AtEnd() == false)
                {
                    DCXELM e = it.Get();
                    Debug.WriteLine(System.Threading.Thread.CurrentThread.Name + ": " + string.Format("{0:X}", e.Tag) + "=" + e.Value.ToString());
                    it.Next();
                }
            }
            public void Query()
            {
                string MyAETitle=System.Environment.GetEnvironmentVariable("COMPUTERNAME");
                DCXREQ requester = new DCXREQ();
                requester.OnQueryResponseRecieved += new IDCXREQEvents_OnQueryResponseRecievedEventHandler(this.OnQueryResponse);
                DCXOBJ o = new DCXOBJ();
                DCXELM e = new DCXELM();
                e.Init((int)DICOM_TAGS_ENUM.QueryRetrieveLevel);
                e.Value = "PATIENT";
                o.insertElement(e);
                e.Init((int)DICOM_TAGS_ENUM.PatientsName);
                e.Value = "*";
                o.insertElement(e);
                e.Init((int)DICOM_TAGS_ENUM.patientID);
                e.Value = "*";
                o.insertElement(e);
                requester.Query(MyAETitle, IS_AE, IS_Host, IS_port, "1.2.840.10008.5.1.4.1.2.1.1", o);
                lock (this)
                {
                    Monitor.Pulse(this);
                }   
            }
        }

        /// <summary>
        /// 
        /// </summary>
        class AccepterHandler
        {
            const int ConnectionLoopInterval = 5;  // seconds
            const int AcceptTimout = 300; // seconds

            public bool done = false;

            public void Accept()
            {
                DCXACC a = new DCXACC();
                int AcceptLoopCounter = AcceptTimout / ConnectionLoopInterval;

                /// Loop for at most AcceptTimout seconds and then 
                while (AcceptLoopCounter-- > 0)
                {
                    string MyAETitle = System.Environment.GetEnvironmentVariable("COMPUTERNAME");
                    a.WaitForConnection(MyAETitle, 104, ConnectionLoopInterval);
                    lock (this)
                    {
                        Monitor.Pulse(this);
                    }
                }
            }
        }

        [Test]
        [Category("Attended")]
        [Description("Test two queries running in two separate threads simultaneously")]
        public void ConcurrentQuery()
        {
            AccepterHandler a1 = new AccepterHandler();
            Thread ta = new Thread(new ThreadStart(a1.Accept));
            ta.Name = "Accepter";
            ta.Start();

            lock (a1)
            {
                Monitor.Wait(a1);
            }

            QueryHandler q1 = new QueryHandler();
            QueryHandler q2 = new QueryHandler();
            Thread t1 = new Thread(new ThreadStart(q1.Query));
            Thread t2 = new Thread(new ThreadStart(q2.Query));
            t1.Name = "Q1";
            t2.Name = "Q2";
            t1.Start();
            t2.Start();
            lock (q1)
            {
                Monitor.Wait(q1);
            }
            lock (q2)
            {
                Monitor.Wait(q2);
            }

            a1.done = true;
        }

        private void req_OnMoveResponseRecieved(bool succeeded)
        {
            unchecked {
            Int32 res;
            res = (int)0x80004004;
            throw new COMException("Cancel it!", res);
            }
        }

        [Test]
        [Category("Query/Retrieve")]
        public void C_MOVE_Cancel()
        {
            try 
            {
                DCXREQ req = new DCXREQ();
                req.OnMoveResponseRecieved+=new IDCXREQEvents_OnMoveResponseRecievedEventHandler(req_OnMoveResponseRecieved);
                req.MoveSeries(MyAETitle, IS_AE, IS_Host, IS_port, MyAETitle, "816300C", "1.2.3.4.5.6.7.917515.2953", "1.2.3.4.5.6.7.917515.2957");
            }
            catch (COMException e)
            {
                Debug.WriteLine(e.ErrorCode + ":" + e.Message);
            }
        }

        private DCXOBJ Build_MPPS_N_CREATE_OBJECT()
        {
            DCXUID uid = new DCXUID();
            string studyInstanceUid = uid.CreateUID(UID_TYPE.UID_TYPE_STUDY);

            DCXOBJ ssas = new DCXOBJ();
            DCXELM e = new DCXELM();
            ///
            /// Scheduled Step Attributes Sequence
            /// 

            e.Init((int)DICOM_TAGS_ENUM.studyInstanceUID);
            e.Value = studyInstanceUid; /// Type 1 (Mandatory)
            ssas.insertElement(e); 

            e.Init((int)DICOM_TAGS_ENUM.ReferencedStudySequence);
            ssas.insertElement(e); /// Type 2 (0 length is OK)
            DCXOBJ obj = new DCXOBJ();

            e.Init((int)DICOM_TAGS_ENUM.AccessionNumber);
            ssas.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.RequestedProcedureID);
            ssas.insertElement(e); /// Type 2 (0 length is OK)
                                   /// 
            e.Init((int)DICOM_TAGS_ENUM.RequestedProcedureDescription);
            ssas.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.ScheduledProcedureStepID);
            ssas.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.ScheduledProcedureStepDescription);
            ssas.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.ScheduledProtocolCodeSequence);
            ssas.insertElement(e); /// Type 2 (0 length is OK)

            DCXOBJIterator sq = new DCXOBJIterator();
            sq.Insert(ssas);

            e.Init((int)DICOM_TAGS_ENUM.ScheduledStepAttributesSequence);
            e.Value = sq;
   
            ///
            /// Performed Procedure Step
            /// 

            DCXOBJ pps = new DCXOBJ();
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.PatientsName);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.patientID);
            pps.insertElement(e); /// Type 2 (0 length is OK)
                                   /// 
            e.Init((int)DICOM_TAGS_ENUM.PatientsBirthDate);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PatientsSex);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.ReferencedPatientSequence);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepID);
            e.Value = 1; /// Some ID of the PPS
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.PerformedStationAETitle);
            e.Value = "RZDCX";
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.PerformedStationName);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedLocation);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepStartDate);
            e.Value = DateTime.Now;
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepStartTime);
            e.Value = DateTime.Now;
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepStatus);
            e.Value = "IN PROGRESS"; /// When creating, the PPS is already in progress
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepDescription);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureTypeDescription);
            pps.insertElement(e); /// Type 2 (0 length is OK) 

            e.Init((int)DICOM_TAGS_ENUM.ProcedureCodeSequence);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepEndDate);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepEndTime);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.Modality);
            e.Value = "CT";
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.StudyID);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedProtocolCodeSequence);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedSeriesSequence);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            return pps;
        }

        private DCXOBJ Build_MPPS_N_SET_OBJECT()
        {
            DCXOBJ ssas = new DCXOBJ();
            DCXELM e = new DCXELM();

            ///
            /// Performed Procedure Step
            /// 

            DCXOBJ pps = new DCXOBJ();
            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepEndDate);
            e.Value = DateTime.Now;
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepEndTime);
            e.Value = DateTime.Now;
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepStatus);
            e.Value = "COMPLETED";
            pps.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureStepDescription);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedProcedureTypeDescription);
            pps.insertElement(e); /// Type 2 (0 length is OK) 

            e.Init((int)DICOM_TAGS_ENUM.ProcedureCodeSequence);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            e.Init((int)DICOM_TAGS_ENUM.PerformedProtocolCodeSequence);
            pps.insertElement(e); /// Type 2 (0 length is OK)

            /// 
            /// Series Sequence
            /// 
            DCXOBJ series_item = new DCXOBJ();
            e.Init((int)DICOM_TAGS_ENUM.PerformingPhysiciansName);
            series_item.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.ProtocolName);
            e.Value = "SOME PROTOCOL";
            series_item.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.OperatorsName);
            series_item.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.seriesInstanceUID);
            e.Value = "1.2.3.4.5.6";
            series_item.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.SeriesDescription);
            series_item.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.RetrieveAETitle);
            series_item.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.ReferencedImageSequence);
            series_item.insertElement(e);

            e.Init((int)DICOM_TAGS_ENUM.ReferencedNonImageCompositeSOPInstanceSequence);
            series_item.insertElement(e);

            DCXOBJIterator series_sq = new DCXOBJIterator();
            series_sq.Insert(series_item);
            e.Init((int)DICOM_TAGS_ENUM.PerformedSeriesSequence);
            e.Value = series_sq;
            pps.insertElement(e);

            return pps;
        }

        [Test]
        [Category("MPPS")]
        [Description("Test MPPS Create and Set workflow")]
        public void MPPS_CreateAndSet()
        {
            DCXREQ req = new DCXREQ();
            DCXOBJ mpps_create_object = Build_MPPS_N_CREATE_OBJECT();
            string sop_instance_uid = req.MPPS_Create(MyAETitle, RIS_AE, RIS_Host, RIS_port, mpps_create_object);
            DCXOBJ mpps_set_object = Build_MPPS_N_SET_OBJECT();
            req.MPPS_Set(MyAETitle, RIS_AE, RIS_Host, RIS_port, mpps_set_object, sop_instance_uid);

        }

        [Test]
        [Category("MPPS")]
        [Category("Error Handling")]
        [Description("Set should fail if not created first")]
        public void MPPS_SetWithoutCreate()
        {
            try
            {
                DCXREQ req = new DCXREQ();
                DCXOBJ mpps_set_object = Build_MPPS_N_SET_OBJECT();
                req.MPPS_Set(MyAETitle, RIS_AE, RIS_Host, RIS_port, mpps_set_object, "1.0.0.0.0.9.1");
            }
            catch (COMException err)
            {
                Assert.That(err.Message.StartsWith("DIMSE Command Failed"));
                Assert.That(err.Message.Contains("0x0112: No such object instance"));
            }
        }

        [Test]
        [Category("Error Handling")]
        [Description("SOP Class Not negotiated error: send MPPS N-CREATE to Storage Server")]
        public void SOPClassNotNegotiated()
        {
            try
            {
                DCXREQ req = new DCXREQ();
                DCXOBJ mpps_create_object = Build_MPPS_N_CREATE_OBJECT();
                req.MPPS_Create(MyAETitle, IS_AE, IS_Host, IS_port, mpps_create_object);
            }
            catch (COMException err)
            {
                Assert.That(err.Message.StartsWith("SOP Class not negotiated"));
            }
        }

        [Test]
        [Description("Connection timeout")]
        public void ConnectionTimeout()
        {
            DCXAPP app = new DCXAPP();
            Assert.That(app.ConnectionTimeout == 15);
            DCXREQ req = new DCXREQ();
            {
                DateTime start = DateTime.Now;
                try
                {
                    req.Echo("DUMMY", "BUMMY", "localhost", 9999 /* A port that is not in use */);
                }
                catch (COMException e)
                {
                    /// this should be the echo failed timeout - peer abprted or never connected
                }
                DateTime end = DateTime.Now;
                TimeSpan elapsed = new TimeSpan(end.Ticks - start.Ticks);
                Assert.That(elapsed.Seconds >= 15 && elapsed.Seconds <= 20);
            }
            app.ConnectionTimeout = 5;
            {
                DateTime start = DateTime.Now;
                try
                {
                    req.Echo("DUMMY", "BUMMY", "localhost", 9999 /* A port that is not in use */);
                }
                catch (COMException e)
                {
                    /// this should be the echo failed timeout - peer abprted or never connected
                }
                DateTime end = DateTime.Now;
                TimeSpan elapsed = new TimeSpan(end.Ticks - start.Ticks);
                Assert.That(elapsed.Seconds >= 5 && elapsed.Seconds <= 10);
            }
            app.ConnectionTimeout = 1;
            {
                DateTime start = DateTime.Now;
                try
                {
                    req.Echo("DUMMY", "BUMMY", "localhost", 9999 /* A port that is not in use */);
                }
                catch (COMException e)
                {
                    /// this should be the echo failed timeout - peer abprted or never connected
                }
                DateTime end = DateTime.Now;
                TimeSpan elapsed = new TimeSpan(end.Ticks - start.Ticks);
                Assert.That(elapsed.Seconds >= 1 && elapsed.Seconds <= 3);
            }
        }
    }
}
