﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using rzdcxLib;
using System.IO;

namespace rzdcxNUnit
{
    [TestFixture]
    [Category("Attended")]
    public class Test_SR
    {
        /// <summary>
        /// Test element value of type PN using the Patient Name tag
        /// </summary>
        [Test]
        public void Test_CreateSimpleSr()
        {
            string sop_class = "1.2.840.10008.5.1.4.1.1.88.22"; /// Enhanced SR
            DCXSR SR = new DCXSR();
            SR.OpenDictionary("StructuredReport.dic");
            SR.CreateNewReport(sop_class, "209074");
            //SR.AddDictContainer("209074", SR_CONTINUITY.SR_SEPARATE);
            SR.AddDictContainer("121070", SR_CONTINUITY.SR_SEPARATE);
            SR.AddDictCodedValue("G-C0E3", "T-35300");
            SR.AddDictNumericValue("G-0380", (float)12.34, null);
      
            /// Check SCOORD
            float[,] scoord = new float[2,1];
            scoord[0,0] = 0.5F;
            scoord[1,0] = 3.9F;
            SR.AddDictSCOORDValue("209239", rzdcxLib.SR_GRAPHIC_TYPE.SR_GT_POINT, scoord);
            SR.UpOneLevel();

            /// Check TCOORD with time offsets
            double[] f64tcoord = new double[1];
            f64tcoord[0] = DateTime.Now.ToFileTime();
            SR.AddDictTCOORDValue("109041", SR_TEMPORAL_RANGE_TYPE.SR_TR_BEGIN, f64tcoord);
            SR.UpOneLevel();

            /// Check TCOORD with sample positions
            uint[] u32tcoord = new uint[2];
            u32tcoord[0] = 0;
            u32tcoord[1] = 4;
            SR.AddDictTCOORDValue("109041", SR_TEMPORAL_RANGE_TYPE.SR_TR_SEGMENT, u32tcoord);
            SR.UpOneLevel();

            /// Check TCOORD with dates
            string[] strtcoord = new string[3];
            strtcoord[0] = DateTime.Now.ToString("yyyyMMddHHmmss.fff");
            strtcoord[1] = DateTime.Now.Add(new TimeSpan(1, 0, 0, 0)).ToString("yyyyMMddHHmmss.fff");
            strtcoord[2] = DateTime.Now.Add(new TimeSpan(2, 0, 0, 0)).ToString("yyyyMMddHHmmss.fff");
            SR.AddDictTCOORDValue("109041", SR_TEMPORAL_RANGE_TYPE.SR_TR_MULTIPOINT, strtcoord);
            SR.UpOneLevel();

            DCXOBJ obj = SR.Object;
            obj.saveFile("sr1.dcm");
        }
    }
}
